
// This is file ADialogs.C.  It implements two classes that manage the
// annotation dialog boxes, described in ADialogs.C.

/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/



#include "TopShell.h"
#include "ADialogs.h"
#include <string.h>
#include <stdlib.h>

#define String X11String
#define Object X11Object

#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/Form.h>
#include <Xm/ScrolledW.h>
#include <Xm/DialogS.h>
#include <Xm/Separator.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/DragDrop.h>
#include <Xm/DropSMgr.h>

#undef String
#undef Object


#define	THIS	FullAnnotationDialog
#define	BASE	UIObject
#define	BASE_CLASSES BASE::desc()
#define	MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES

DEFINE_REDUCED_CLASS(FullAnnotationDialog,1,"$Header: /home/cellar/ginsburg/browser/RCS/ADialogs.C,v 2.1 1996/05/17 20:12:23 ginsburg Exp $",NULL)


// Redefine for second class:
#undef THIS
#define	THIS	SmallAnnotationDialog

DEFINE_REDUCED_CLASS(SmallAnnotationDialog,1,"$Header: /home/cellar/ginsburg/browser/RCS/ADialogs.C,v 2.1 1996/05/17 20:12:23 ginsburg Exp $",NULL)


bool FullAnnotationDialog::isEqual(const Object& p) const
// Test two objects for equality
{
#ifdef FULL_NIHCL
  return p.isSpecies(classDesc) && *this==castdown(p);
#else
  return this == &p;
#endif
}

bool SmallAnnotationDialog::isEqual(const Object& p) const
// Test two objects for equality
{
#ifdef FULL_NIHCL
  return p.isSpecies(classDesc) && *this==castdown(p);
#else
  return this == &p;
#endif
}


// Create a new annotation creation/modification dialog with the
// specified parent.

FullAnnotationDialog::FullAnnotationDialog(Widget parent)
:cancelCB((CBType)NULL),
 deleteCB((CBType)NULL)
{
  // Create the dialog and set its title:
  shell =
    XtCreatePopupShell("annotation", xmDialogShellWidgetClass, parent, NULL, 0);

  XtVaSetValues(shell,
		XmNtitle, "Annotation Settings",
		XmNallowShellResize, False,
		NULL);

  // Create a form manager for the dialog:
  form = XtVaCreateWidget("form", xmFormWidgetClass, shell,
 			  XmNfractionBase, 5,
 			  XmNnoResize, True,
 			  NULL);

  // Save, close, and delete buttons:
  save_button = XtVaCreateWidget("OK", xmPushButtonWidgetClass, form,
				 XmNleftAttachment, XmATTACH_POSITION,
				 XmNleftPosition, 1,
				 XmNrightAttachment, XmATTACH_POSITION,
				 XmNrightPosition, 2,
				 XmNbottomAttachment, XmATTACH_FORM,
				 XmNbottomOffset, 10,
				 NULL);

  cancel_button = XtVaCreateWidget("Cancel", xmPushButtonWidgetClass, form,
				   XmNleftAttachment, XmATTACH_POSITION,
				   XmNleftPosition, 3,
				   XmNrightAttachment, XmATTACH_POSITION,
				   XmNrightPosition, 4,
				   XmNbottomAttachment, XmATTACH_FORM,
				   XmNbottomOffset, 10,
				   NULL);

  // WM_DELETE also cancels:
  register_cb(cancel_button, XmNactivateCallback,
	      (CBType)&FullAnnotationDialog::cancel_buttonCB, NULL);
  register_wm_cb(shell, delete_window_atom,
		 (CBType)&FullAnnotationDialog::cancel_buttonCB, NULL);

  delete_button = XtVaCreateWidget("Delete", xmPushButtonWidgetClass, form,
				   XmNleftAttachment, XmATTACH_POSITION,
				   XmNleftPosition, 3,
				   XmNrightAttachment, XmATTACH_POSITION,
				   XmNrightPosition, 4,
				   XmNbottomAttachment, XmATTACH_FORM,
				   XmNbottomOffset, 10,
				   XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET,
				   XmNtopWidget, save_button,
				   NULL);

  // Set the callbacks for save and delete buttons:
  register_cb(save_button, XmNactivateCallback,
	      (CBType)&FullAnnotationDialog::callCB, &saveCB);
  register_cb(delete_button, XmNactivateCallback,
	      (CBType)&FullAnnotationDialog::callCB, &deleteCB);

  // Create a separator to set the buttons apart from the work area:
  separator = XtVaCreateWidget("separator", xmSeparatorWidgetClass, form,
			       XmNleftAttachment, XmATTACH_FORM,
			       XmNrightAttachment, XmATTACH_FORM,
			       XmNbottomAttachment, XmATTACH_WIDGET,
			       XmNbottomWidget, save_button,
			       XmNbottomOffset, 10,
			       NULL);


  // Create the annotation text display/editor:
  text_display = XmCreateScrolledText(form, "text_display", NULL, 0);

  // Since text_display is a scrolled text, constraints must be set on
  // its parent, the scrolled window:
  XtVaSetValues(XtParent(text_display),
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		XmNbottomAttachment, XmATTACH_WIDGET,
		XmNbottomWidget, separator,
		XmNbottomOffset, 10,
		XmNleftOffset, 10,
		XmNrightOffset, 10,
		NULL);

  // Configure the text widget itself:
  XtVaSetValues(text_display,
		XmNeditable, True,
		XmNeditMode, XmMULTI_LINE_EDIT,
		XmNcolumns, 40,
		XmNrows, 8,
		XmNwordWrap, True,
		NULL);

  // A title for the annotation text:
  annotation_label = XtVaCreateWidget("Annotation:", xmLabelWidgetClass, form,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNleftOffset, 10,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, XtParent(text_display),
				      XmNbottomOffset, 10,
				      NULL);

  // A text line for the annotation name:
  name_text = XtVaCreateWidget("name_text", xmTextWidgetClass, form,
			       XmNeditable, True,
			       XmNeditMode, XmSINGLE_LINE_EDIT,
			       XmNcolumns, 40,
			       XmNleftAttachment, XmATTACH_FORM,
			       XmNleftOffset, 10,
			       XmNbottomAttachment, XmATTACH_WIDGET,
			       XmNbottomWidget, annotation_label,
			       XmNbottomOffset, 20,
			       NULL);

  // A title for the annotation name:
  name_label = XtVaCreateWidget("Name:", xmLabelWidgetClass, form,
				XmNleftAttachment, XmATTACH_FORM,
				XmNleftOffset, 10,
				XmNbottomAttachment, XmATTACH_WIDGET,
				XmNbottomWidget, name_text,
				XmNbottomOffset, 10,
				XmNtopAttachment, XmATTACH_FORM,
				XmNtopOffset, 10,
				NULL);

  // A title for the hyperlink:
  link_label = XtVaCreateWidget("link_label", xmLabelWidgetClass, form,
				XmNleftAttachment, XmATTACH_WIDGET,
				XmNleftWidget, name_text,
				XmNleftOffset, 20,
				XmNtopAttachment, XmATTACH_FORM,
				XmNtopOffset, 10,
				XtVaTypedArg, XmNlabelString, XtRString,
				"Hyperlink\nDestination:",
				sizeof("Hyperlink\nDestination:"),
				NULL);

  // A label to describe the hyperlink destination:
  link_dest = XtVaCreateWidget("link_dest", xmLabelWidgetClass, form,
			       XmNleftAttachment, XmATTACH_OPPOSITE_WIDGET,
			       XmNleftWidget, link_label,
			       XmNleftOffset, 10,
			       XmNtopAttachment, XmATTACH_WIDGET,
			       XmNtopWidget, link_label,
			       XmNtopOffset, 10,
			       NULL);

  // A frame to hold the link point:
  link_frame = XtVaCreateWidget("link_frame", xmFrameWidgetClass, form,
				XmNleftAttachment, XmATTACH_WIDGET,
				XmNleftWidget, link_label,
				XmNleftOffset, 10,
				XmNtopAttachment, XmATTACH_FORM,
				XmNtopOffset, 10,
				//XmNbottomAttachment, XmATTACH_WIDGET,
				//XmNbottomWidget, annotation_label,
				//XmNbottomOffset, 20,
				XmNshadowType, XmSHADOW_IN,
				NULL);

  // The actual source/destination of annotation hyperlink drags:
  link_point = XtVaCreateWidget("link_point", xmLabelWidgetClass, link_frame,
				NULL);

  // Set the link point icon to the "unlinked" annotation icon
  assign_label_pixmap(link_point, annotation_icon_pixmaps);

  Arg args[10];
  int n = 0;

  Atom importList[1];

  // existing annotation drops accepted:
  importList[0] = existing_annotation_atom;

  // Set the docwin_context of XtWindow(link_point) to this
  XSaveContext(dpy, (XID)link_point, docwin_context, (XPointer)this);

  // Register the drop site:
  XtSetArg(args[n], XmNimportTargets, importList); n++;
  XtSetArg(args[n], XmNnumImportTargets, XtNumber(importList)); n++;
  XtSetArg(args[n], XmNanimationStyle, XmDRAG_UNDER_SHADOW_IN); n++;
  XtSetArg(args[n], XmNdropSiteOperations, XmDROP_LINK); n++;
  XtSetArg(args[n], XmNdropProc, icon_dropCB); n++;

  XmDropSiteRegister(link_point, args, n);

  // For use with XtManageChildren:
  children = new Widget[9];
  children[0] = name_label;
  children[1] = name_text;
  children[2] = annotation_label;
  children[3] = separator;
  children[4] = link_label;
  children[5] = link_frame;
  children[6] = save_button;
  children[7] = delete_button;
  children[8] = link_dest;
}


// Deallocate resources

FullAnnotationDialog::~FullAnnotationDialog()
{
  XtDestroyWidget(shell);
  delete [] children;
}


// Make the dialog visible

void FullAnnotationDialog::popup_and_manage()
{
  // Manage the dialog widgets:
  XtManageChild(link_point);

  if (cancelCB)
    XtManageChild(cancel_button);

  XtManageChildren(children, 9);

  XtManageChild(text_display);
  XtManageChild(form);

  // Popup the dialog
  XtPopup(shell, XtGrabNonexclusive);

  // Set the focus to the text widget
  XmProcessTraversal(name_text, XmTRAVERSE_CURRENT);
}


// Set the various work area fields in the dialog and specify parent
// members to call on different inputs.  annotation gives the
// annotation text, name give the name, and ok_title, cancel_title specify
// the titles for their respective buttons.  new_parent and the
// callbacks are used in response to user input:
//
//  new_saveCB - called when the user clicks "OK"
//  new_deleteCB - delete this object, i.e. when the user clicks "Cancel"
//  new_dropCB - called when a new annotation is dropped on the link
//    point
//
// Note that this form of set_fields is used for the creation of new
// annotations.  A different form configures the dialog for modifying
// existing annotations.

void FullAnnotationDialog::set_fields(const char *annotation, const char *name,
				      const char *ok_title, const char *cancel_title,
				      Annotation *new_parent,
				      CBType new_saveCB, CBType new_deleteCB,
				      CBType new_dropCB)
{
  // Set the parent and callbacks:
  parent = new_parent;
  drop_and_linkCB = new_dropCB;
  cancelCB = (CBType)NULL;
  saveCB = new_saveCB;
  deleteCB = new_deleteCB;

  // Use two buttons, save_button and delete_button:
  XtVaSetValues(form, XmNfractionBase, 5, NULL);
  XtVaSetValues(delete_button,
		XmNleftPosition, 3,
		XmNrightPosition, 4,
		NULL);

  // Set the text strings:
  XmTextSetString(text_display, (char*)annotation);
  XmTextSetString(name_text, (char*)name);

  // Set the button titles:
  set_labelString(save_button, ok_title);
  set_labelString(delete_button, cancel_title);
}


// Set the various work area fields in the dialog and specify parent
// members to call on different inputs.  annotation gives the
// annotation text, name give the name, and ok_title, cancel_title,
// delete_title specify the titles for their respective buttons.
// new_parent and the callbacks are used in response to user input:
//
//  saveCB - called when the user clicks "OK"
//  new_cancelCB - called when the user clicks "Cancel"
//  deleteCB - delete this object, i.e. when the user clicks "Remove"
//  new_dropCB - called when a new annotation is dropped on the link
//    point
//
// Note that this form of set_fields is used for modifying existing
// annotations.  An alternative form configures the dialog for
// creating new annotations.

void FullAnnotationDialog::set_fields(const char *annotation, const char *name,
				      const char *ok_title, const char *cancel_title,
				      const char *delete_title, Annotation *new_parent,
				      CBType new_saveCB, CBType new_cancelCB,
				      CBType new_deleteCB, CBType new_dropCB)
{
  // Set the parent and callbacks:
  parent = new_parent;
  drop_and_linkCB = new_dropCB;
  cancelCB = new_cancelCB;
  saveCB = new_saveCB;
  deleteCB = new_deleteCB;

  // Configure the dialog for three buttons; move delete_button to
  // make room for cancel_button:
  XtVaSetValues(form, XmNfractionBase, 7, NULL);
  XtVaSetValues(delete_button,
		XmNleftPosition, 5,
		XmNrightPosition, 6,
		NULL);

  // Set the text strings:
  XmTextSetString(text_display, (char*)annotation);
  XmTextSetString(name_text, (char*)name);

  // Set the button titles:
  set_labelString(save_button, ok_title);
  set_labelString(cancel_button, cancel_title);
  set_labelString(delete_button, delete_title);
}


// Create a new annotation text dialog as the child of the specified widget.

SmallAnnotationDialog::SmallAnnotationDialog(Widget parent)
{
  // Create a form dialog and set the title:
  form = XmCreateFormDialog(parent, "an_form", NULL, 0);

  XtVaSetValues(XtParent(form),
		XmNtitle, "Annotation",
		NULL);

  XtVaSetValues(form,
		XmNfractionBase, 5,
		NULL);

  // The dialog has a single "OK" button, a separator, and a scrolled
  // text widget:
  ok_button = XtVaCreateWidget("OK", xmPushButtonWidgetClass, form,
			       XmNleftAttachment, XmATTACH_POSITION,
			       XmNleftPosition, 2,
			       XmNrightAttachment, XmATTACH_POSITION,
			       XmNrightPosition, 3,
			       XmNbottomAttachment, XmATTACH_FORM,
			       XmNbottomOffset, 10,
			       NULL);

  separator = XtVaCreateWidget("separator", xmSeparatorWidgetClass, form,
				  XmNleftAttachment, XmATTACH_FORM,
				  XmNrightAttachment, XmATTACH_FORM,
				  XmNbottomAttachment, XmATTACH_WIDGET,
				  XmNbottomWidget, ok_button,
				  XmNbottomOffset, 10,
				  NULL);

  text_display = XmCreateScrolledText(form, "text_display", NULL, 0);

  // Set the form constraints on the parent of the text (the scrolled
  // window):
  XtVaSetValues(XtParent(text_display),
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		XmNtopAttachment, XmATTACH_FORM,
		XmNbottomAttachment, XmATTACH_WIDGET,
		XmNbottomWidget, separator,
		XmNbottomOffset, 10,
		XmNleftOffset, 10,
		XmNrightOffset, 10,
		XmNtopOffset, 10,
		NULL);

  // Configure the text widget itself:
  XtVaSetValues(text_display,
		XmNeditable, False,
		XmNeditMode, XmMULTI_LINE_EDIT,
		XmNcolumns, 40,
		XmNrows, 8,
		XmNwordWrap, True,
		NULL);

  // Register an "OK" callback:
  register_cb(ok_button, XmNactivateCallback,
	      (CBType)&UIObject::unmanageCB, (XtPointer)form);
}


// Destroy the dialog

SmallAnnotationDialog::~SmallAnnotationDialog()
{
  XtDestroyWidget(form);
}


// Install a new annotation text string

void SmallAnnotationDialog::set_fields(const char *annotation)
{
  XmTextSetString(text_display, (char*)annotation);
}


// Make the dialog visible

void SmallAnnotationDialog::popup_and_manage()
{
  XtManageChild(ok_button);
  XtManageChild(separator);
  XtManageChild(text_display);
  XtManageChild(form);		// also pops up the dialog

  // Set the focus to the text widget
  XmProcessTraversal(text_display, XmTRAVERSE_CURRENT);
}


// Set the link icon to the unlinked state, then install the specified
// drag translation address for the "link_drag" action.

void FullAnnotationDialog::set_link(char *link_addr)
{
  // Set the link point pixmap to "unlinked"
  assign_label_pixmap(link_point, annotation_icon_pixmaps);

  // Clear the destination text
  set_labelString(link_dest, "");

  // Install the new drag translation:
  static const char link_translation_format[] =
    "<Btn2Down>: link_drag(%s)\n";

  // Use the specified drag ActionProc:
  char *link_translations = new char[strlen(link_translation_format) + 1*16];
  sprintf(link_translations, link_translation_format, link_addr);

  XtOverrideTranslations(link_point, XtParseTranslationTable(link_translations));

  delete [] link_translations;
}


// Set the link icon to the linked state, using the specified link
// name and page name.

void FullAnnotationDialog::set_link(char *link_name, const char *page)
{
  // Remove the drag translation
  XtOverrideTranslations(link_point, XtParseTranslationTable("<Btn2Down>: \n"));

  // Set the link point icon to "linked"
  assign_label_pixmap(link_point, destination_icon_pixmaps);

  // Set the link text display:
  char *name_string = new char[strlen(link_name) + 40];

  // If link_name is empty, use "(unnamed)" rather than the actual name:
  if (*link_name)
    sprintf(name_string, "Linked to\n\"%s\",\nPage %s", link_name, page);
  else
    sprintf(name_string, "Linked to\n(unnamed),\nPage %s", page);

  set_labelString(link_dest, name_string);

  delete [] name_string;
}


// Receive annotations dropped onto the link point

void FullAnnotationDialog::icon_dropCB(Widget widget, XtPointer, XmDropProcCallbackStruct *cbs)
{
  XmDropTransferEntryRec transferEntries[1];
  static Arg args[10];
  static int n;

  n = 0;

  // Accept only link operations:
  if ((cbs->dropAction != XmDROP) || (cbs->operation != XmDROP_LINK))
    {
      XtSetArg(args[n], XmNtransferStatus, XmTRANSFER_FAILURE); n++;
    }
  else
    {
      // Retrieve the object pointer from the widget's docwin_context:
      DocWindow *this_obj = NULL;

      if ((XFindContext(dpy, (XID)widget, docwin_context,
			(XPointer*)&this_obj) == 0) &&
	  this_obj)
	{
	  // Setup the transfer:
	  transferEntries[0].target = existing_annotation_atom;
	  transferEntries[0].client_data = (XtPointer)this_obj;
	  XtSetArg(args[n], XmNdropTransfers, transferEntries); n++;
	  XtSetArg(args[n], XmNnumDropTransfers, XtNumber(transferEntries)); n++;
	  XtSetArg(args[n], XmNtransferProc, TransferProc); n++;
	}
      else
	{
	  // No context; probably a drag from somewhere else
	  XtSetArg(args[n], XmNtransferStatus, XmTRANSFER_FAILURE); n++;
	}
    }

  // Start the transfer
  XmDropTransferStart(cbs->dragContext, args, n);
}


// Handle the actual conversion by calling parent->*drop_and_linkCB.
// format argument might be int rather than int*; documentation is
// sketchy

void FullAnnotationDialog::TransferProc(Widget widget, XtPointer closure,
					Atom *, Atom *type, XtPointer value,
					unsigned long *, int*)
{
  // Retrieve the "this" pointer
  FullAnnotationDialog * const this_obj = (FullAnnotationDialog*)closure;

  if (*type == existing_annotation_atom)
    {
      // value is a pointer to the integer index of the annotation
      const int selected_icon = ((int*)value)[0];

      // Inform the parent annotation of the new link:
      (((UIObject*)(this_obj->parent))->*(this_obj->drop_and_linkCB))
	(widget, (XtPointer)selected_icon, NULL);
    }
}
