//  This may look like C, but it is really -*- C++ -*-

// This is file ADialogs.h.  It describes two classes that manage the
// annotation dialog boxes, implemented in ADialogs.C.

/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/



#ifndef ADialogs_H
#define ADialogs_H


#include "UIObject.h"

#define String X11String
#define Object X11Object

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>

#undef String
#undef Object


class Annotation;


// A class to handle the annotation creation/modification dialog:

class FullAnnotationDialog: public UIObject {
  DECLARE_REDUCED_MEMBERS(FullAnnotationDialog);

private:
  // The dialog components:
  Widget shell, form, name_label, name_text, annotation_label, text_display;
  Widget separator, link_label, link_frame, link_point, link_dest;
  Widget save_button, cancel_button, delete_button;

  // A list of form children for rapid management
  Widget *children;

  // Drop callback and TransferProc for receiving annotation drops:
  static void icon_dropCB(Widget, XtPointer, XmDropProcCallbackStruct*);
  static void TransferProc(Widget, XtPointer, Atom*, Atom*, XtPointer, unsigned long *, int*);

  // Cancel the annotation modification by calling cancelCB if it
  // exists, otherwise delete the new annotation by calling deleteCB:
  void cancel_buttonCB(Widget widget, XtPointer client_data, XtPointer call_data)
    { if (cancelCB) (((UIObject*)parent)->*cancelCB)(widget, client_data, call_data);
    else if (deleteCB) (((UIObject*)parent)->*deleteCB)(widget, this, call_data); }

  // Call the member of parent specified by *client_data
  void callCB(Widget widget, CBType *client_data, XtPointer call_data)
    { (((UIObject*)parent)->*(*client_data))(widget, NULL, call_data); }

  // The Annotation object responsible for this dialog, along with
  // methods to be invoked through this object:
  Annotation *parent;
  CBType drop_and_linkCB, cancelCB, saveCB, deleteCB;

public:
  // Create/destroy dialogs:
  FullAnnotationDialog(Widget);
  ~FullAnnotationDialog();

  // Get the shell widget
  Widget get_shell() { return shell; };

  // Set the various text fields and callback methods:
  void set_fields(const char *, const char *, const char *, const char *,
		  Annotation*, CBType, CBType, CBType);
  void set_fields(const char *, const char *, const char *, const char *, const char *,
		  Annotation*, CBType, CBType, CBType, CBType);

  // Get the annotation text or name:
  char *get_annotation() { return XmTextGetString(text_display); };
  char *get_name() { return XmTextGetString(name_text); };

  // Make the dialog visible
  void popup_and_manage();

  // Set the link icon to the unlinked state, then install the
  // specified drag translation address for the "link_drag" action
  void set_link(char *);

  // Set the link icon to the linked state, using the specified link
  // name and page name
  void set_link(char *, const char *);

public:
  bool operator==(const FullAnnotationDialog& a) const
    { return (this == &a); }
  bool operator!=(const FullAnnotationDialog& a) const
    { return !(*this==a); }
  virtual bool isEqual(const Object&) const;
#ifdef FULL_NIHCL
  virtual const Class* species() const
    { return &classDesc; }
#endif
};



// A class to handle the annotation text dialog:

class SmallAnnotationDialog: public UIObject {
  DECLARE_REDUCED_MEMBERS(SmallAnnotationDialog);

private:
  // Dialog elements
  Widget form, text_display, ok_button, separator;

public:
  // Create/destroy the dialog:
  SmallAnnotationDialog(Widget);
  ~SmallAnnotationDialog();

  // Set the annotation text
  void set_fields(const char *);

  // Return the shell widget
  Widget get_shell() { return XtParent(form); };

  // Make the dialog visible
  void popup_and_manage();

public:
  bool operator==(const SmallAnnotationDialog& a) const
    { return (this == &a); }
  bool operator!=(const SmallAnnotationDialog& a) const
    { return !(*this==a); }
  virtual bool isEqual(const Object&) const;
#ifdef FULL_NIHCL
  virtual const Class* species() const
    { return &classDesc; }
#endif
};



#endif
