
// This is file Bitmap.C.  It implements a class for handling one-bit
// pixmaps, defined in Bitmap.h.


/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/



#include "Bitmap.h"
#include <nihcl/String.h>


#define	THIS	Bitmap
#define	BASE	ReducedObj
#define	BASE_CLASSES BASE::desc()
#define	MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES

DEFINE_REDUCED_CLASS(Bitmap,1,"$Header: /a/cellar62/home/ginsburg/browser/RCS/Bitmap.C,v 2.0 1996/03/13 20:53:11 ginsburg Exp $",NULL)


bool Bitmap::isEqual(const Object& p) const
// Test two objects for equality
{
#ifdef FULL_NIHCL
  return p.isSpecies(classDesc) && *this==castdown(p);
#else
  return this == &p;
#endif
}


// Create a new bitmap from a file.

Bitmap::Bitmap(const String &filename)
{
  int hotx, hoty;

  // Read the bitmap file:
  unsigned int bm_error =
    XReadBitmapFile(dpy, DefaultRootWindow(dpy), (const char*)filename,
		    &width, &height, &bitmap, &hotx, &hoty);

  if (bm_error != BitmapSuccess)
    cerr << "Warning: thumbnail \"" << filename <<
      "\" could not be loaded." << endl;
}


// Create a Bitmap object with the specified pixmap and dimensions.
// The pixmap will be deallocated when the object is destroyed.

Bitmap::Bitmap(Pixmap new_bitmap, const unsigned int new_width, const unsigned int new_height)
:width(new_width),
 height(new_height)
{
  bitmap = new_bitmap;
}


// Deallocate the bitmap.

Bitmap::~Bitmap()
{
  XFreePixmap(dpy, bitmap);
}


// Create and return a new pixmap with the full depth of the screen,
// copying the bitmap with the specified colors.

Pixmap Bitmap::deepen(unsigned long fg, unsigned long bg)
{
  // Create the new pixmap:
  Pixmap deep = XCreatePixmap(dpy, DefaultRootWindow(dpy),
			      width, height, DefaultDepthOfScreen(scrn));

  if (!deep)
    cerr << "Warning: could not create deep pixmap" << endl;

  // Create a GC:
  GC gc = XCreateGC(dpy, deep, 0, NULL);

  if (!gc)
    cerr << "Warning: could not create deep pixmap gc" << endl;

  // Set the colors in the GC:
  XSetForeground(dpy, gc, fg);
  XSetBackground(dpy, gc, bg);

  // Copy the bitmap
  XCopyPlane(dpy, bitmap, deep, gc, 0, 0, width, height, 0, 0, 1);

  XFreeGC(dpy, gc);

  return deep;
}
