//  This may look like C, but it is really -*- C++ -*-

// This is file Bitmap.h.  It describes a class for handling one-bit
// pixmaps, implemented in Bitmap.C.

/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/



#ifndef Bitmap_H
#define Bitmap_H


#include "ReducedObj.h"
#include "TopShell.h"

#define String X11String
#define Object X11Object

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#undef String
#undef Object


class String;
class Thumbnail;
class BitmapThumbnail;
class AntiAliasedThumbnail;


class Bitmap: public ReducedObj {
  DECLARE_REDUCED_MEMBERS(Bitmap);

  friend Thumbnail;
  friend BitmapThumbnail;
  friend AntiAliasedThumbnail;

protected:
  Pixmap bitmap;		// the actual pixmap
  unsigned int width, height;	// the bitmap dimensions

public:
  Bitmap(const String&);	// create a bitmap from a file
  // Create a bitmap from an existing pixmap
  Bitmap(Pixmap, const unsigned int, const unsigned int);
  ~Bitmap();			// deallocate the bitmap

  // Cast by returning the pixmap
  operator Pixmap () const { return bitmap; }
  // Return a full-depth pixmap
  Pixmap deepen(unsigned long, unsigned long);
  // deepen with no args uses black, white:
  Pixmap deepen()
    { return deepen(BlackPixelOfScreen(scrn), WhitePixelOfScreen(scrn)); }

  // Access the dimensions:
  unsigned int get_width() const { return width; }
  unsigned int get_height() const { return height; }


public:
  bool operator==(const Bitmap& a) const
    { return (this == &a); }
  bool operator!=(const Bitmap& a) const
    { return !(*this==a); }
  virtual bool isEqual(const Object&) const;
#ifdef FULL_NIHCL
  virtual const Class* species() const
    { return &classDesc; }
#endif
};

#endif
