//  This may look like C, but it is really -*- C++ -*-

// This is file DocWindow.h.  It describes a class for managing
// document-level windows as well as the PostScript file itself.


/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/



#ifndef DocWindow_H
#define DocWindow_H


// 	$Id: DocWindow.h,v 2.1 1996/05/17 20:12:58 ginsburg Exp ginsburg $	


#include "UIObject.h"
#include <nihcl/String.h>
#include <nihcl/OrderedCltn.h>
#include <nihcl/Stack.h>

#include <sys/wait.h>		// for pid_t
#include <sys/stat.h>		// for stat

#define String X11String
#define Object X11Object

#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/DropSMgr.h>

extern "C" {
#include "ghostview/ps.h"
}

#undef String
#undef Object


#define DOC_BORDER  20
#define DOC_SHADOW  10

#define DOC_LEFT_MARGIN  DOC_BORDER
#define DOC_RIGHT_MARGIN  DOC_BORDER + DOC_SHADOW
#define DOC_TOTAL_MARGIN  DOC_LEFT_MARGIN + DOC_RIGHT_MARGIN

#define MAX_WINDOW_WIDTH  1000
#define MAX_WINDOW_HEIGHT  1000

#define SCROLLBAR_THICKNESS  25

#define THUMBNAIL_DPI  10

// Characters to ignore when matching page names:
#define IGNORE_THESE_CHARS  "\""
// (FrameMaker surrounds page numbers with ".)

enum {OS_AUTO = 0, OS_PORTRAIT = 1, OS_LANDSCAPE = 2, OS_INVERTED = 3, OS_SEASCAPE = 4};

enum {CB_PROPERTIES = 0, CB_TEXT = 1, CB_HYPERLINK = 2, CB_DELETE = 3};

enum {REGION_STATUS, REGION_THUMBNAILS};

struct document;
class ScanControl;
class SortedCltn;
class Annotation;
class ExportDialog;
class TopShell;

class DocWindow: public UIObject {
  DECLARE_REDUCED_MEMBERS(DocWindow);

  friend Annotation;

private:
  Widget shell, main_window, menubar, filemenu, pagemenu, viewmenu;
  Widget gvform, gvarea, gvscrolled, gvhscroll, gvvscroll, ghostview;
  Widget mag_dialog,  msgform, msglabel;
  Widget page_dialog, next_page_button, prev_page_button;
  Widget annotation_icon, hyperlink_icon;

  Widget orientation_buttons[5];
  int active_orientation, auto_orientation, physical_orientation;

  ScanControl *scan;
  OrderedCltn annotations;
  TopShell *parent;

  ExportDialog *save_dialog, *print_dialog;

  struct document *doc;
  FILE *psfile;
  const String psfilename;
  int current_page;
  documentmedia *current_media;
  int current_bounding_box[4];
  float full_page_xdpi, full_page_ydpi;
  Dimension full_page_width, full_page_height;
  Dimension gvwidth, gvheight, areawidth, areaheight;
  GC gc;
  int button_down_x, button_down_y;
  int x_slider_size, y_slider_size, x_inc, y_inc, x_page_inc, y_page_inc;
  int x_value, y_value, orig_x_value, orig_y_value;
  int x_min, x_max, y_min, y_max;
  Boolean clicked_in_gvarea, annotations_modified;
  int gv_x_margin, gv_y_margin;
  Boolean size_established, allow_annotations;
  char *icon_drag_addr;
  struct stat file_stats;
  time_t file_modification_time;

  void change_pageCB(Widget, XtPointer, XtPointer);
  void write_messageCB(Widget, XtPointer, XtPointer);
  void handle_gs_messageCB(Widget, XtPointer, XtPointer);
  void set_magnificationCB(Widget, XtPointer, XtPointer);
  void set_magnification_directlyCB(Widget, int, XtPointer);
  void layout_page_geometryAP(Widget, XConfigureEvent*, char*[], int*);
  void goto_pageCB(Widget, XtPointer, XtPointer);
  void scrollCB(Widget, int, XmScrollBarCallbackStruct*);
  void expose_hackAP(Widget, XEvent*, char*[], int*);
  void fit_widthCB(Widget, XtPointer, XtPointer);
  void fit_heightCB(Widget, XtPointer, XtPointer);
  void fit_to_windowCB(Widget, XtPointer, XtPointer);
  void set_orientationCB(Widget, int, XtPointer);
  void init_dragAP(Widget, XEvent*, char*[], int);
  void icon_dragAP(Widget, XEvent*, char*[], int);
  static void handle_dropCB(Widget, XtPointer, XmDropProcCallbackStruct*);
  void delete_annotationCB(Widget, Annotation*, XtPointer);
  void exportCB(Widget, ExportDialog*, XtPointer);
  void process_keyAP(Widget, XKeyEvent*, char*[], int);
  void process_keyEH(Widget, XtPointer, XKeyEvent*, Boolean*);
  void toggle_regionCB(Widget, int, XmToggleButtonCallbackStruct*);
  void toggle_annotationsCB(Widget, XtPointer, XmToggleButtonCallbackStruct*);
  void get_infoCB(Widget, XtPointer, XtPointer);

  static Boolean DragConvertProc(Widget, Atom*, Atom*, Atom*, XtPointer*,
				 unsigned long*, int *, unsigned long *,
				 char *, XtRequestId*);
  static Boolean ButtonConvertProc(Widget, Atom*, Atom*, Atom*, XtPointer*,
				   unsigned long*, int *, unsigned long *,
				   int, XtRequestId*);

  static void TransferProc(Widget, XtPointer, Atom*, Atom*,
			   XtPointer, unsigned long *, int*);


  static int names_approx_equal(const char *, const char *);
  int virtual_to_real_page(int virtual_page) const;
  int page_label_to_virtual_page(const String &) const;
  void move_gv_and_annotations(Position, Position);
  void export_ps(const int, FILE*);
  void change_gv_orientation(const int);
  void send_current_page_ps();
  static void convert_to_virtual_coords(float&, float&, const int);

  void pointer_clickEH(Widget, XtPointer, XButtonEvent*, Boolean*);
  void pointer_motionEH(Widget, XtPointer, XButtonEvent*, Boolean*);

public:
  DocWindow(Widget parent, FILE *, const String &, CBType, TopShell *, int&,
	    pid_t&);
  ~DocWindow();

  void draw_page(const int);
  void update_scan(const int);
  void draw_page(int, Pixmap&);
  int get_current_page() { return current_page; }
  void jump_to_pos(float, float);
  int *bounding_box_of_page(const int);

  void add_drag_translations(Widget);
  void write_message(const char *);

  Stack full_dialog_stack, annotation_dialog_stack;

  Boolean bookmarks_modified;

  // The popup menu widget ID's for the Annotation class:
  Widget annotation_popup_no_buttons, annotation_popup_text_button;
  Widget annotation_popup_link_button, annotation_popup_both_buttons;

  CBType popup_callbacks[4];
  Annotation *callback_root;
  void call_popup_callbackCB(Widget, int, XtPointer);

  void save_file(const int, const char *);
  void print_file(const int, const char *);

  static int os_to_gv_orientation(const int);
  documentmedia *get_page_media(const int);
  int *get_page_boundingbox(const int);
  int get_page_orientation(const int);

public:
  bool operator==(const DocWindow& a) const
    { return (this == &a); }
  bool operator!=(const DocWindow& a) const
    { return !(*this==a); }
  virtual bool isEqual(const Object&) const;
#ifdef FULL_NIHCL
  virtual const Class* species() const
    { return &classDesc; }
#endif
};

#endif
