//  This may look like C, but it is really -*- C++ -*-

//  This is file ExportDialog.h.  It describes a class for the "Export
//  PostScript"/"Print" dialogs, defined in ExportDialog.C.

/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/



#ifndef ExportDialog_H
#define ExportDialog_H

#include "UIObject.h"

#define String X11String
#define Object X11Object

#include <X11/Intrinsic.h>
#include <Xm/FileSB.h>

#undef String
#undef Object


class DocWindow;

#define ENABLE_LEFT_COLUMN  (1 << 2)
#define ENABLE_CENTER_COLUMN  (1 << 1)
#define ENABLE_RIGHT_COLUMN  (1 << 0)

#define TYPE_SAVE_DIALOG  1
#define TYPE_PRINT_DIALOG  2

#define EXPORT_PERSISTENT_BOOKMARKS  0
#define EXPORT_ENTIRE_DOCUMENT  1
#define EXPORT_TEMPORARY_BOOKMARKS 2


class ExportDialog: public UIObject {
  DECLARE_REDUCED_MEMBERS(ExportDialog);

private:
 // Widgets for the dialog, form manager, option menu, text label
 // (either "in" or "using"), text input line, browse button and file
 // selection dialog.  The last two are used only when type ==
 // TYPE_SAVE_DIALOG:
  Widget shell, form, option_menu, in_label;
  Widget text_line, browse_button, filebox;

  // Widgets for a (horizontal) separator, two action buttons, and the
  // buttons comprising the option menu:
  Widget separator, ok_button, cancel_button;
  Widget menu_widgets[3];

  const int type;		// set to TYPE_SET_DIALOG or TYPE_PRINT_DIALOG
  int column;			// maintains the currently selected
				// option-menu button
  char *default_filename;	// default filename or print command
  
  DocWindow *parent_obj;	// parent DocWindow object
  Widget *children;		// set to an array of form children

  // Callbacks for the option menu, "OK" button, and file selection dialog:
  static void option_menuCB(Widget, int, XtPointer);
  void okCB(Widget, XtPointer, XtPointer);
  void fileokCB(Widget, XtPointer, XmFileSelectionBoxCallbackStruct*);

public:
  ExportDialog(Widget, DocWindow*, const int, const char *);
  ~ExportDialog();

  void popup_and_manage();	// set default values, then popup the dialog
  void set_available(const int); // sensitive/desensitive menu items

public:
  bool operator==(const ExportDialog& a) const
    { return (this == &a); }
  bool operator!=(const ExportDialog& a) const
    { return !(*this==a); }
  virtual bool isEqual(const Object&) const;
#ifdef FULL_NIHCL
  virtual const Class* species() const
    { return &classDesc; }
#endif
};

#endif
