//  This may look like C, but it is really -*- C++ -*-

// This is file MessageBox.h.  It describes a class for managing
// message dialogs, implemented in MessageBox.C.

/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/



#ifndef MessageBox_H
#define MessageBox_H


// Identifying values for the dialog buttons.  Note that the actual
// button titles may vary:
#define MBNoButton 0
#define MBOkButton 1
#define MBCancelButton 2
#define MBHelpButton 4


#include "UIObject.h"


class MessageBox: public UIObject {
  DECLARE_REDUCED_MEMBERS(MessageBox);

private:
  Widget message_box;		// the dialog box
  int answer;			// store the user's response

  // Called when a button is pressed or the dialog popped down
  void responseCB(Widget, XtPointer, XtPointer);

public:

  // Create a new message box
  MessageBox(Widget, int, char *, char **, int, int);

  // Wait for user input, then return the button pressed
  int get_response();

  // Delete the dialog box:
  ~MessageBox()
    { XtDestroyWidget(message_box); }

public:
  bool operator==(const MessageBox& a) const
    { return (this == &a); }
  bool operator!=(const MessageBox& a) const
    { return !(*this==a); }
  virtual bool isEqual(const Object&) const;
#ifdef FULL_NIHCL
  virtual const Class* species() const
    { return &classDesc; }
#endif
};

#endif
