//  This may look like C, but it is really -*- C++ -*-

#ifndef ReducedObj_H
#define ReducedObj_H

/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/


#include <nihcl/Object.h>


inline int min(int x, int y) {return ((x<=y) ? x : y);}
inline float min(float x, float y) {return ((x<=y) ? x : y);}
inline unsigned int min(unsigned int x, unsigned int y) {return ((x<=y) ? x : y);}

inline int max(int x, int y) {return ((x>y) ? x : y);}
inline float max(float x, float y) {return ((x>y) ? x : y);}
inline unsigned int max(unsigned int x, unsigned int y) {return ((x>y) ? x : y);}



#ifndef DECLARE_REDUCED_MEMBERS
#define DECLARE_REDUCED_MEMBERS(classname) \
private: \
	static Class classDesc; \
public: \
	DECLARE_CASTDOWN(classname) \
	static const Class* desc()  { return &classDesc; } \
	static classname* readFrom(OIOin& strm) { return castdown(desc()->readFrom(strm)); } \
	static classname* readFrom(OIOifd& fd) { return castdown(desc()->readFrom(fd)); } \
	virtual const Class* isA() const; \
	virtual Object* shallowCopy() const; \
	virtual void* _castdown(const Class&) const; \
protected: \
	void deepenVBase(); \
	void storeVBaseOn(OIOofd&) const; \
	void storeVBaseOn(OIOout&) const \


/*private: \
static Object* reader(OIOin& strm); \
static Object* reader(OIOifd& fd) \ */
#endif


#ifndef DEFINE_REDUCED_CLASS

#define _DEFINE_REDUCED_CLASS(classname) \
Object* classname::shallowCopy() const	{ return (Object*)new classname(*this); } \

#define _DEFINE_REDUCED_ABSTRACT_CLASS(classname) \
extern const int NIHCL_RDABSTCLASS; \
Object* classname::shallowCopy() const	{ derivedClassResponsibility("shallowCopy"); return nil; } \

#define DEFINE_REDUCED_CLASS(classname,version,identification,initor) \
_DEFINE_REDUCED_CLASS(classname) \
_DEFINE_CLASS_ALWAYS(classname,version,identification,initor) \
_DEFINE_CASTDOWN(classname) \

#define DEFINE_REDUCED_ABSTRACT_CLASS(classname,version,identification,initor) \
_DEFINE_REDUCED_ABSTRACT_CLASS(classname) \
_DEFINE_CLASS_ALWAYS(classname,version,identification,initor) \
_DEFINE_CASTDOWN(classname) \

#endif


class ReducedObj: public VIRTUAL Object
{
  DECLARE_MEMBERS(ReducedObj);

#ifdef FULL_NIHCL  
protected:
  virtual void storer(OIOofd&) const;
  virtual void storer(OIOout&) const;
#endif
public:
  ReducedObj(){};
  bool operator==(const ReducedObj& a) const
    { return (this == &a); }
  bool operator!=(const ReducedObj& a) const
    { return !(*this==a); }
  virtual int compare(const Object&) const;
#ifdef FULL_NIHCL
  virtual Object* copy() const { return shallowCopy(); }
  virtual void deepenShallowCopy() {}
#endif
  virtual unsigned hash() const
    { return 0; }
  virtual bool isEqual(const Object&) const;
#ifdef FULL_NIHCL
  virtual void printOn(ostream& strm =cout) const;
  virtual const Class* species() const;
#endif
};

#endif
