
/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/



#include "ScanControl.h"
#include "DocWindow.h"
#include "Thumbnail.h"
#include <nihcl/Integer.h>
#include "TopShell.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/wait.h>

#define String X11String
#define Object X11Object

#include <X11/Intrinsic.h>
#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/DragDrop.h>
#include <Xm/DropSMgr.h>

extern "C" {
#include "Generic.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
}

#undef String
#undef Object

#include "icons/thumbnail"


// XmDROP_SITE_VALID was accidentally ommited in early
// versions of Xm/DropSMgr.h (?):
#ifndef XmDROP_SITE_VALID
#define XmDROP_SITE_VALID XmVALID_DROP_SITE
#endif


#define	THIS	ScanControl
#define	BASE	UIObject
#define	BASE_CLASSES BASE::desc()
#define	MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES

DEFINE_REDUCED_CLASS(ScanControl,1,"$Header: /home/cellar/ginsburg/browser/RCS/ScanControl.C,v 2.3 1996/07/05 15:42:32 ginsburg Exp ginsburg $",NULL)


bool ScanControl::isEqual(const Object& p) const
// Test two objects for equality
{
#ifdef FULL_NIHCL
  return p.isSpecies(classDesc) && *this==castdown(p);
#else
  return this == &p;
#endif
}


// A script to begin rendering thumbnails:
static char rendering_prolog[] =
"%! PS-Adobe-2.0
%%  This script is based on pstoppm.ps from Ghostscript 2.5.2:

%% Program for reading a .ps file and writing out a PPM file.
%% For Ghostscript 2.5.2.


%% Modified by Adam Ginsburg 3/31/96:
%%   output written to stderr for DeckView 
%% Modified by L. Peter Deutsch 9/10/92:
%%   internal procedures didn't use `bind';
%%   grestoreall undid selection of PPM device.
%% Modified by L. Peter Deutsch 4/6/92:
%%   Ghostscript 2.4 requires all 8 primary colors to be in the palette.
%% Modified by L. Peter Deutsch 1/17/92:
%%   the palette for makeimagedevice is now a string, not an array.
%% Modified by L. Peter Deutsch 9/24/91:
%%   allow starting page number to be specified.
%% Modified by L. Peter Deutsch 7/7/91 to keep track of page count
%%   in a way that gets around save and restore.
%% Modified by L. Peter Deutsch 11/07/90
%%   to use filename.ppm for the first page, renamed to filename.1ppm
%%   with subsequent pages .2ppm, etc. if more than one page.
%% Modified by Henry Minsky 11/03/90
%%   for each showpage, it writes out a ppm file with name filename.ppm.N
%%   where N increments each showpage, starting at 1
%% Modified by L. Peter Deutsch -- Aladdin Enterprises -- 08/25/90 --
%%   converted from a one-shot program to a utility package,
%%   designed to be used from an interactive terminal.
%% Modified by L. Peter Deutsch -- Aladdin Enterprises -- 08/02/90
%% Modified on 08/02/90 for using the CORRECT color map.
%% Modified 06/26/90 for a color file
%% Original version by Phillip Conrad - Perfect Byte, Inc.
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%% Define the directory for holding the PPM information

/ppmdict 30 dict def
ppmdict begin 

  /Horz_dpi 72 def		% defaults to screen density
  /Vert_dpi 72 def

  /Palette1 <ff 00> def

  % Redefine copypage and showpage appropriately.
  /copypage { ppmdict begin .stderr Device writeppmfile WrotePage 0 1 put end } bind userdict begin odef end
  /showpage { copypage erasepage initgraphics } bind userdict begin odef end

  % Redefine grestoreall so it doesn't undo the device selection
  /grestoreall /grestoreall load def
  /grestoreall { ppmdict begin   grestoreall   Device setdevice   end } bind userdict begin odef end

  /Convert\
   { /Palette exch def\
     /FileNameIn exch def\
\
     /WrotePage 1 string def\
\
     /ScaleX Horz_dpi 72 div def\
     /ScaleY Vert_dpi 72 div def\
\
     /Width 85 Horz_dpi mul 5 add 10 div cvi def\
     /Height 11 Vert_dpi mul def\
\
     [ScaleX 0.0 0.0 ScaleY neg 0.0 Height]\
     Width Height Palette makeimagedevice\
       /Device exch def\
     Device setdevice\
\
     FileNameIn end run\
     ppmdict begin\
     WrotePage 0 get 0 eq { showpage } if\
     end \
\
} bind def

end				% ppmdict

%%%%%% Define the user-callable procedures

/ppm1run\
 { ppmdict begin   Palette1 Convert\
 } bind def

%  call as (filename) ppm1run

/setres \
{\
    ppmdict begin\
    /Vert_dpi exch def\
    /Horz_dpi exch def\
    end \
} bind def

%  call as <Horizontal DPI> <Vertical DPI> setres
";


// Is point in column i; i in (0..2)?
inline Boolean ScanControl::in_column_i(const int x, const int y,
					const int column)
{
  return
    !((y < SCAN_TOP_MARGIN) || (y > height - SCAN_BOTTOM_MARGIN)
      || (x < (SCAN_LEFT_RIGHT_MARGIN
	       + column * (SCAN_GUTTER_WIDTH + bitmap_width)))
      || (x > (SCAN_LEFT_RIGHT_MARGIN
	       + column * (SCAN_GUTTER_WIDTH + bitmap_width) + bitmap_width)));
}


// Is point in gutter i; i in (0..1)?
inline Boolean ScanControl::in_gutter_i(const int x, const int y,
					const int column)
{
  return
    !((y < SCAN_TOP_MARGIN) || (y > height - SCAN_BOTTOM_MARGIN)
      || (x < (SCAN_LEFT_RIGHT_MARGIN + bitmap_width
	       + column * (SCAN_GUTTER_WIDTH + bitmap_width)))
      || (x > (SCAN_LEFT_RIGHT_MARGIN
	       + (column + 1) * (SCAN_GUTTER_WIDTH + bitmap_width))));
}


static XErrorHandler default_handler;
static Boolean alloc_failed;


static int badalloc_handler(Display *dpy, XErrorEvent *error)
{
  if (error->error_code == BadAlloc)
    alloc_failed = True;

  if (alloc_failed)
    return 0;

  return default_handler(dpy, error);
}


int ScanControl::read_pbmraw(FILE *pbmfile, int &width, int &height,
			     Pixmap &new_bitmap)
{
  char buf[256];

  // PORTRAIT or LANDSCAPE only; if orientation is PORTRAIT, do
  // nothing, otherwise rotate 90 degrees clockwise
  const int current_orientation =
    (parent->get_page_orientation(page_num) == PORTRAIT)
      ? OS_PORTRAIT : OS_LANDSCAPE;
  const int desired_orientation =
    (orientation == OS_AUTO) ? current_orientation : orientation;

  int orientation_difference = desired_orientation - current_orientation;

  // Ensure that orientation_difference is between 0 and 3:
  if (orientation_difference < 0)
    orientation_difference += 4;

  default_handler = XSetErrorHandler(badalloc_handler);
  alloc_failed = False;

  // Check magic number:
  fgets(buf, 256, pbmfile);
  if (!((buf[0] == 'P') && (buf[1] == '4')))
    {
      cerr << "Error: invalid PBM format: " << buf << endl;
      new_bitmap = XmUNSPECIFIED_PIXMAP;
      return PBM_ERROR;
    }

  // skip the comment
  fgets(buf, 256, pbmfile);

  // read the dimensions
  fgets(buf, 256, pbmfile);
  if (2 != sscanf(buf, "%d %d\n", &width, &height))
    {
      cerr << "Error: could not find PBM dimensions" << endl;
      new_bitmap = XmUNSPECIFIED_PIXMAP;
      return PBM_ERROR;
    }

  // pbm_width, height indicate dimensions of the bitmap to expect
  // from Ghostscript; width, height are the dimensions of the pixmap
  // to create, which might be rotated:

  int pbm_width = width;
  int pbm_height = height;

  if (orientation_difference % 2 == 1)
    swap(width, height);	// swap dimensions if necessary

  // Create a bitmap for the thumbnail:
  new_bitmap = XCreatePixmap(dpy, DefaultRootWindow(dpy), width, height, 1);

  XSync(dpy, False);

  if (alloc_failed)
    {
      parent->write_message("Warning: unable to allocate thumbnail pixmap");
      new_bitmap = XmUNSPECIFIED_PIXMAP;
      return PBM_ERROR;
    }

  // Create an image to load the actual values into:
  XImage *xi =
    XGetImage(dpy, new_bitmap, 0, 0, width, height, AllPlanes, ZPixmap);

  if (!xi)
    {
      parent->write_message("Warning: unable to allocate XImage in thumbnail loading");
      XFreePixmap(dpy, new_bitmap);
      new_bitmap = XmUNSPECIFIED_PIXMAP;
      return PBM_ERROR;
    }

  int width_count = 0, height_count = 0, pixel_count = 0;
  static const unsigned char masks[] = {128, 64, 32, 16, 8, 4, 2, 1};

  const int line_length = (pbm_width + 7) / 8;

  unsigned char *bytes = new unsigned char[line_length];

  for (int y = 0; y < pbm_height; y++)
    {
      if (orientation_difference == 0)
	{
	  fread(bytes, sizeof(*bytes), line_length, pbmfile);
	  for (int x = 0; x < line_length; x++)
	    for(int j = 0; j < 8; j++)
	      XPutPixel(xi, x * 8 + j, y, (bytes[x] & masks[j]) ? 1 : 0);
	}
      else if (orientation_difference == 1)
	{
	  fread(bytes, sizeof(*bytes), line_length, pbmfile);
	  for (int x = 0; x < line_length; x++)
	    for(int j = 0; j < 8; j++)
	      XPutPixel(xi, y, x * 8 + j, (bytes[x] & masks[j]) ? 1 : 0);
	}
      else if (orientation_difference == 2)
	{
	  fread(bytes, sizeof(*bytes), line_length, pbmfile);
	  for (int x = 0; x < line_length; x++)
	    for(int j = 0; j < 8; j++)
	      XPutPixel(xi, (pbm_width / 2) - (x * 8 + j), y,
			(bytes[x] & masks[j]) ? 1 : 0);
	}
      else if (orientation_difference == 3)
	{
	  fread(bytes, sizeof(*bytes), line_length, pbmfile);
	  for (int x = 0; x < line_length; x++)
	    for(int j = 0; j < 8; j++)
	      XPutPixel(xi, y, (pbm_width / 2) - (x * 8 + j),
			(bytes[x] & masks[j]) ? 1 : 0);
	}

      // Check for pending events every fourth scan line:
      if ((y % 4 == 0) && (XtIMXEvent & XtAppPending(TopShell::app)))
	XtAppProcessEvent(TopShell::app, XtIMXEvent);
    }

  XPutImage(dpy, new_bitmap, bitmap_gc, xi, 0, 0, 0, 0, width, height);
  XDestroyImage(xi);

  // Restore the default error handler
  XSetErrorHandler(default_handler);

  delete [] bytes;
  return PBM_SUCCESS;
}


int ScanControl::read_pbmraw(FILE *pbmfile, int &width, int &height,
			     Pixmap &pixmap, Pixmap &active)
{
  default_handler = XSetErrorHandler(badalloc_handler);

  // PORTRAIT or LANDSCAPE only; if orientation is PORTRAIT, do
  // nothing, otherwise rotate 90 degrees clockwise
  const int current_orientation =
    (parent->get_page_orientation(page_num) == PORTRAIT)
      ? OS_PORTRAIT : OS_LANDSCAPE;
  const int desired_orientation =
    (orientation == OS_AUTO) ? current_orientation : orientation;

  int orientation_difference = desired_orientation - current_orientation;

  // Ensure that orientation_difference is between 0 and 3:
  if (orientation_difference < 0)
    orientation_difference += 4;

  // Create masks and lookup tables for bitmap decoding and antialiasing:

  // masks: 1 << 7, 1 << 6, 1 << 5, ..., 1 << 0
  static const unsigned char masks[] = {128, 64, 32, 16, 8, 4, 2, 1};

  // mini_lut: for 0..15, how many 1's in the binary representation?
  static const unsigned char mini_lut[] =
    {0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4};

  // lut_masks: mask the four half-nibbles in a byte:
  // 00000011, 00001100, 00110000, 11000000
  static const unsigned char lut_masks[] = {3 << 0, 3 << 2, 3 << 4, 3 << 6};
  static unsigned char *lut = NULL;

  if (!lut)
    {
      // Create and initialize the lookup table:
      lut = new unsigned char[4 * 65536];
      if (!lut)
	{
	  cerr << "DeckView: unable to allocate anti-aliasing table" << endl;
	  pixmap = active = XmUNSPECIFIED_PIXMAP;
	  return PBM_ERROR;
	}

      unsigned char key, byte1, byte2;

      for (int i = 0; i < 65536; i++)
	{
	  // Extract the two bytes from the counter:
	  byte1 = i & 0x00ff;
	  byte2 = (i >> 8) & 0x00ff;
	  for (int j = 3; j >= 0; j--)
	    {
	      key = ((((byte1 & lut_masks[j]) >> (j << 1)) << 2) |
		     ((byte2 & lut_masks[j]) >> (j << 1)));
	      lut[i * 4 + (3 - j)] = mini_lut[key];
	    }
	}
    }

  // Read in the file:

  char buf[256];

  // Check magic number:
  fgets(buf, 256, pbmfile);
  if (!((buf[0] == 'P') && (buf[1] == '4')))
    {
      cerr << "Error: invalid PBM format: " << buf << endl;
      pixmap = active = XmUNSPECIFIED_PIXMAP;
      return PBM_ERROR;
    }

  // skip the comment
  fgets(buf, 256, pbmfile);

  // read the dimensions
  fgets(buf, 256, pbmfile);
  if (2 != sscanf(buf, "%d %d\n", &width, &height))
    {
      cerr << "Error: could not find PBM dimensions" << endl;
      pixmap = active = XmUNSPECIFIED_PIXMAP;
      return PBM_ERROR;
    }

  // pbm_width, height indicate dimensions of the bitmap to expect
  // from Ghostscript; width, height are the dimensions of the pixmap
  // to create, which might be rotated:

  int pbm_width = width;
  int pbm_height = height;

  if (orientation_difference % 2 == 1)
    swap(width, height);	// swap dimensions if necessary

  // Create pixmaps for the thumbnail:
  pixmap =
    XCreatePixmap(dpy, DefaultRootWindow(dpy), width / 2, height / 2,
		  DefaultDepthOfScreen(scrn));
  active =
    XCreatePixmap(dpy, DefaultRootWindow(dpy), width / 2, height / 2,
		  DefaultDepthOfScreen(scrn));

  XSync(dpy, False);

  if (alloc_failed)
    {
      parent->write_message("Warning: pixmap allocation failed in thumbnail loading");
      XFreePixmap(dpy, pixmap);
      XFreePixmap(dpy, active);

      pixmap = active = XmUNSPECIFIED_PIXMAP;
      return PBM_ERROR;
    }

  // Use XImages for the actual manipulation:
  XImage *pm_xi =
    XGetImage(dpy, pixmap, 0, 0, width / 2, height / 2, AllPlanes, ZPixmap);
  XImage *ac_xi =
    XGetImage(dpy, pixmap, 0, 0, width / 2, height / 2, AllPlanes, ZPixmap);

  if (!pm_xi || !ac_xi)
    {
      parent->write_message("Warning: XImage allocation failed in thumbnail loading");

      if (pm_xi) XDestroyImage(pm_xi);
      if (ac_xi) XDestroyImage(pm_xi);

      XFreePixmap(dpy, pixmap);
      XFreePixmap(dpy, active);

      pixmap = active = XmUNSPECIFIED_PIXMAP;
      return PBM_ERROR;
    }

  unsigned long colors[5], active_colors[5];
  colors[0] = TopShell::appdata.dim_page_background;
  colors[1] = TopShell::appdata.level_one;
  colors[2] = TopShell::appdata.level_two;
  colors[3] = TopShell::appdata.level_three;
  colors[4] = BlackPixelOfScreen(scrn);

  active_colors[0] = WhitePixelOfScreen(scrn);
  active_colors[1] = TopShell::appdata.level_one;
  active_colors[2] = TopShell::appdata.level_two;
  active_colors[3] = TopShell::appdata.level_three;
  active_colors[4] = BlackPixelOfScreen(scrn);

  int width_count = 0, height_count = 0, pixel_count = 0;

  const int line_length = (pbm_width + 7) / 8;

  unsigned char *bytes1 = new unsigned char[line_length];
  unsigned char *bytes2 = new unsigned char[line_length];

  for (int y = 0; y < pbm_height / 2; y++)
    {
      // Read in two lines:
      fread(bytes1, sizeof(*bytes1), line_length, pbmfile);
      fread(bytes2, sizeof(*bytes2), line_length, pbmfile);

      if (orientation_difference == 0)
	for (int x = 0; x < line_length; x++)
	  {
	    const unsigned char * const pixels =
	      lut + 4 * ((bytes1[x] << 8) | bytes2[x]);
	    for (int j = 0; j < 4; j++)
	      {
		if (width / 2 <= x * 4 + j)
		  continue;	// Be careful not to write off the
				// edge of the image
		XPutPixel(pm_xi, x * 4 + j, y, colors[*(pixels + j)]);
		XPutPixel(ac_xi, x * 4 + j, y, active_colors[*(pixels + j)]);
	      }
	  }
      else if (orientation_difference == 1)
	for (int x = 0; x < line_length; x++)
	  {
	    const unsigned char * const pixels =
	      lut + 4 * ((bytes1[x] << 8) | bytes2[x]);
	    for (int j = 0; j < 4; j++)
	      {
		if (height / 2 <= x * 4 + j)
		  continue;	// Be careful not to write off the
				// edge of the image
		XPutPixel(pm_xi, pbm_height / 2 - 1 - y, x * 4 + j,
			  colors[*(pixels + j)]);
		XPutPixel(ac_xi, pbm_height / 2 - 1 - y, x * 4 + j,
			  active_colors[*(pixels + j)]);
	      }
	  }
      else if (orientation_difference == 2)
	for (int x = 0; x < line_length; x++)
	  {
	    const unsigned char * const pixels =
	      lut + 4 * ((bytes1[x] << 8) | bytes2[x]);
	    for (int j = 0; j < 4; j++)
	      {
		if (0 > (pbm_width / 2 - 1) - (x * 4 + j))
		  continue;	// Be careful not to write off the
				// edge of the image
		XPutPixel(pm_xi, (pbm_width / 2 - 1) - (x * 4 + j),
			  pbm_height / 2 - 1 - y, colors[*(pixels + j)]);
		XPutPixel(ac_xi, (pbm_width / 2 - 1) - (x * 4 + j),
			  pbm_height / 2 - 1 - y,
			  active_colors[*(pixels + j)]);
	      }
	  }
      else if (orientation_difference == 3)
	for (int x = 0; x < line_length; x++)
	  {
	    const unsigned char * const pixels =
	      lut + 4 * ((bytes1[x] << 8) | bytes2[x]);
	    for (int j = 0; j < 4; j++)
	      {
		if (0 > (pbm_width / 2 - 1) - (x * 4 + j))
		  continue;	// Be careful not to write off the
				// edge of the image
		XPutPixel(pm_xi, y, (pbm_width / 2 - 1) - (x * 4 + j),
			  colors[*(pixels + j)]);
		XPutPixel(ac_xi, y, (pbm_width / 2 - 1) - (x * 4 + j),
			  active_colors[*(pixels + j)]);
	      }
	  }

      // Check for pending events every fourth scan line:
      if ((y % 4 == 0) && (XtIMXEvent & XtAppPending(TopShell::app)))
	XtAppProcessEvent(TopShell::app, XtIMXEvent);
    }

  XPutImage(dpy, pixmap, pixmap_gc, pm_xi, 0, 0, 0, 0, width / 2, height / 2);
  XPutImage(dpy, active, pixmap_gc, ac_xi, 0, 0, 0, 0, width / 2, height / 2);

  XDestroyImage(pm_xi);
  XDestroyImage(ac_xi);

  delete [] bytes1;
  delete [] bytes2;

  // Return the processed values:
  width /= 2;
  height /= 2;

  // Restore the error handler
  XSetErrorHandler(default_handler);

  return PBM_SUCCESS;
}


ScanControl::ScanControl(DocWindow *new_parent, Widget parent_form,
			 document *new_doc, OrderedCltn *old_bookmarks,
			 const char *new_psfilename)
:parent(new_parent),
 doc(new_doc),
 orientation(OS_AUTO),
 rendering_in_progress(False),
 psfilename(strdup(new_psfilename))
{
  current_scan_page = held_page = 0;

  // Load the bitmaps:
  bitmap_width = 0;
  bitmap_height = 0;

  int bitmap_page_width = 0, bitmap_page_height = 0;

  // Image the document:
  int stdout_desc[2], stdin_desc[2], stderr_desc[2];
  if (pipe(stdout_desc) || pipe(stdin_desc) || pipe(stderr_desc))
    cerr << "Error: could not create pipe" << endl;

  pid = fork();
  if (pid == -1)
    cerr << "Error: could not fork" << endl;
  else if (pid == 0)
    {
      // child
      close(stdout_desc[0]); // child uses descriptors[1]
      close(stdin_desc[1]);
      close(stderr_desc[0]);

      // Set stdout, stderr, stdin to the pipe
      dup2(stdin_desc[0], 0);
      dup2(stdout_desc[1], 1);
      dup2(stderr_desc[1], 2);
      close(stdin_desc[0]);
      close(stdout_desc[1]);
      close(stderr_desc[1]);

      // exec Ghostscript:
      execl(TopShell::appdata.gs_path,
	    "gs", "-sOutputFile=-", "-q", "-sDEVICE=pbmraw", "-dSAFER",
	    "-dNOPAUSE", "-dNODISPLAY", NULL);

      perror("DeckView: exec failed");
      exit(1);
    }
  // parent
  close(stdin_desc[0]); // parent uses descriptors[1]
  close(stdout_desc[1]);
  close(stderr_desc[1]);

  // Use the prolog to render the pages:
  char use_stdin[] = ".stdin run\n";
  write(stdin_desc[1], use_stdin, (sizeof use_stdin) - 1);
  write(stdin_desc[1], rendering_prolog, (sizeof rendering_prolog) - 1);

  static char commands[300];
  const int multiple = TopShell::appdata.antialias ? 2 : 1;
  sprintf(commands, "%d %d setres\n(%s) ppm1run\n",
	  TopShell::appdata.thumbnail_dpi * multiple,
	  TopShell::appdata.thumbnail_dpi * multiple,
	  psfilename);

  write(stdin_desc[1], commands, strlen(commands));

  close(stdin_desc[1]);

  // Create the GC's (use a dummy bitmap to set the depth)
  Pixmap temp_bitmap = XCreatePixmap(dpy, DefaultRootWindow(dpy), 1, 1, 1);
  bitmap_gc = XCreateGC(dpy, temp_bitmap, 0, NULL);
  XFreePixmap(dpy, temp_bitmap);
  // Antialiased pixmaps have the same depth as the root window:
  pixmap_gc = XCreateGC(dpy, RootWindowOfScreen(scrn), 0, NULL);

  XSetForeground(dpy, bitmap_gc, BlackPixelOfScreen(scrn));
  XSetBackground(dpy, bitmap_gc, WhitePixelOfScreen(scrn));

  // Read the resulting PBM files:
  pbmfile = fdopen(stderr_desc[0], "r");
  page_num = 0;

  rendering_in_progress = True;
  input_id =
    XtAppAddInput(TopShell::app, stderr_desc[0], (XtPointer)XtInputReadMask,
		  read_page, (XtPointer)this);

  for(int i = 0; i < doc->numpages; i++)
    {
      int *bounding_box = parent->bounding_box_of_page(i);

      const int defined_width =
	bounding_box[URX] - bounding_box[LLX];
      const int defined_height =
	bounding_box[URY] - bounding_box[LLY];

      delete [] bounding_box;

      const unsigned int new_width =
	(unsigned int)((float)defined_width / 72.0 *
		       (float)(TopShell::appdata.thumbnail_dpi));
      const unsigned int new_height =
	(unsigned int)((float)defined_height / 72.0 *
		       (float)(TopShell::appdata.thumbnail_dpi));

      Thumbnail *tn;

      const int this_orientation =
	(parent->get_page_orientation(i) == PORTRAIT) ?
	  OS_PORTRAIT : OS_LANDSCAPE;

      // May not be needed, depending on whether bounding_box reflects the
      // correctly oriented dimensions:

      /*if (this_orientation == OS_LANDSCAPE)
	swap(new_width, new_height);*/

      if (TopShell::appdata.antialias)
	tn = new AntiAliasedThumbnail(new_width, new_height,
				      doc->pages[i].label, i,
				      this_orientation);
      else
	tn = new BitmapThumbnail(new_width, new_height,
				 doc->pages[i].label, i,
				 this_orientation);

      thumbnails.add(*tn);

      bitmap_width = max(bitmap_width, (int)new_width);
      bitmap_height = max(bitmap_height, (int)new_height);
    }

  for (int i = 0; i < old_bookmarks->size(); i++)
    {
      const int page = (int)(((Integer*)((*old_bookmarks)[i]))->value());

      if ((page < 0) || (page >= doc->numpages))
	{
	  cerr << "Warning: bookmark on page " << page <<
	    " is out-of-bounds (ignored)." << endl;
	  continue;
	}

      if (TopShell::appdata.antialias)
	persistent_marks.add(*((AntiAliasedThumbnail*)(thumbnails[page])));
      else
	persistent_marks.add(*((BitmapThumbnail*)(thumbnails[page])));
    }
  delete old_bookmarks;
  
  char *expose_addr = register_ap("scan_expose",
				  (APType)&ScanControl::scan_exposeAP);

  char *input_addr = register_ap("scan_input",
				 (APType)&ScanControl::scan_inputAP);


  char *scan_translation_format =
    "<Expose>:   scan_expose(%s)\n\
     <Btn1Down>: scan_input(downclick, %s)\n\
     <Btn1Motion>: scan_input(motion, %s)\n\
     <Btn1Up>: scan_input(upclick, %s)\n\
     <Btn2Down>: scan_input(dragclick, %s)\n";

  char *parent_translation_format =
    "<Btn1Motion>: scan_input(motion, %s)\n\
     <Btn1Up>: scan_input(upclick, %s)\n";


  char *scan_translations =
    new char[strlen(scan_translation_format) + 6*16];

  char *parent_translations =
    new char[strlen(parent_translation_format) + 2*16];

  sprintf(scan_translations, scan_translation_format,
	  expose_addr, input_addr, input_addr, input_addr, input_addr);

  sprintf(parent_translations, parent_translation_format,
	  input_addr, input_addr);

  delete [] expose_addr;
  delete [] input_addr;

  const int scan_width = (3 * bitmap_width + 2 * SCAN_LEFT_RIGHT_MARGIN
			  + 2 * SCAN_GUTTER_WIDTH);

  scan_control_frame =
    XtVaCreateManagedWidget("scan_control_frame", xmFrameWidgetClass,
			    parent_form,
			    XmNtopAttachment, XmATTACH_FORM,
			    XmNleftAttachment, XmATTACH_FORM,
			    XmNbottomAttachment, XmATTACH_FORM,
			    XmNrightAttachment, XmATTACH_OPPOSITE_FORM,
			    XmNrightOffset,
			    -(scan_width + 8 * SHADOW_THICKNESS +
			      6 * FRAME_MARGINS),
			    XmNshadowType, XmSHADOW_IN,
			    XmNshadowThickness, SHADOW_THICKNESS,
			    NULL);

  scan_control_form =
    XtVaCreateManagedWidget("scan_control_form", xmFormWidgetClass,
			    scan_control_frame,
			    XmNfractionBase, 100,
			    XmNtranslations,
			    XtParseTranslationTable(parent_translations),
			    NULL);

  // Positions:
  const int column_size = 2 * SHADOW_THICKNESS + 2 * FRAME_MARGINS + bitmap_width;
  const int left_frame_left_offset = SCAN_LEFT_RIGHT_MARGIN;
  const int left_frame_right_offset = left_frame_left_offset + column_size;
  const int center_frame_left_offset =
    left_frame_right_offset + SCAN_GUTTER_WIDTH;
  const int center_frame_right_offset = center_frame_left_offset + column_size;
  const int right_frame_left_offset =
    center_frame_right_offset + SCAN_GUTTER_WIDTH;
  const int right_frame_right_offset = right_frame_left_offset + column_size;

  left_title =
    XtVaCreateManagedWidget("left_title", xmLabelWidgetClass,
			    scan_control_form,
			    XmNtopAttachment, XmATTACH_FORM,
			    XmNleftAttachment, XmATTACH_FORM,
			    XmNleftOffset, left_frame_left_offset,
			    XmNrightAttachment, XmATTACH_OPPOSITE_FORM,
			    XmNrightOffset, -left_frame_right_offset,
			    XtVaTypedArg, XmNlabelString, XtRString,
			    "Persistent\nBookmarks",
			    sizeof("Persistent\nBookmarks"),
			    XmNalignment, XmALIGNMENT_CENTER,
			    NULL);


  static char label_string[100];
  sprintf(label_string, "Page %s", doc->pages[current_scan_page].label);
  XmString new_label = XmStringCreateSimple(label_string);

  Dimension height;
  XtVaGetValues(left_title,
		XmNheight, &height,
		NULL);

  // A hack to prevent gratuitous resizing of the document window:

  Widget wrapper =
    XtVaCreateManagedWidget("wrapper", genericWidgetClass,
			    scan_control_form,
			    XmNy, 0,
			    XmNx, center_frame_left_offset,
			    XmNwidth, column_size,
			    XmNheight, height,
			    XmNborderWidth, 0,
			    NULL);

  page_display =
    XtVaCreateManagedWidget("page_display", xmLabelWidgetClass,
			    wrapper,
			    XmNlabelString, new_label,
			    XmNalignment, XmALIGNMENT_CENTER,
			    XmNlabelString,
			    XmStringCreateSimple("Temporary\nBookmarks"),
			    NULL);

  XmStringFree(new_label);

  right_title =
    XtVaCreateManagedWidget("right_title", xmLabelWidgetClass,
			    scan_control_form,
			    XmNtopAttachment, XmATTACH_FORM,
			    XmNleftAttachment, XmATTACH_FORM,
			    XmNleftOffset, right_frame_left_offset,
			    XmNrightAttachment, XmATTACH_OPPOSITE_FORM,
			    XmNrightOffset, -right_frame_right_offset,
			    XtVaTypedArg, XmNlabelString, XtRString,
			    "Temporary\nBookmarks",
			    sizeof("Temporary\nBookmarks"),
			    NULL);

  XtTranslations compiled_parent_translations =
    XtParseTranslationTable(parent_translations);

  XtOverrideTranslations(left_title, compiled_parent_translations);
  XtOverrideTranslations(right_title, compiled_parent_translations);
  XtOverrideTranslations(page_display, compiled_parent_translations);

  left_frame =
    XtVaCreateManagedWidget("left_frame", xmFrameWidgetClass,
			    scan_control_form,
			    XmNshadowType, XmSHADOW_IN,
			    XmNshadowThickness, SHADOW_THICKNESS,
			    XmNmarginWidth, FRAME_MARGINS,
			    XmNmarginHeight, FRAME_MARGINS,
			    XmNleftAttachment, XmATTACH_FORM,
			    XmNleftOffset, left_frame_left_offset,
			    XmNrightAttachment, XmATTACH_OPPOSITE_FORM,
			    XmNrightOffset, -left_frame_right_offset,
			    XmNtopAttachment, XmATTACH_WIDGET,
			    XmNtopWidget, left_title,
			    XmNbottomAttachment, XmATTACH_FORM,
			    XmNbottomOffset, SCAN_BOTTOM_MARGIN,
			    NULL);

  Dimension title_height;

  XtVaGetValues(left_title,
		XmNheight, &title_height,
		NULL);

  center_frame =
    XtVaCreateManagedWidget("center_frame", xmFrameWidgetClass,
			    scan_control_form,
			    XmNshadowType, XmSHADOW_ETCHED_IN,
			    XmNshadowThickness, SHADOW_THICKNESS,
			    XmNmarginWidth, FRAME_MARGINS,
			    XmNmarginHeight, FRAME_MARGINS,
			    XmNleftAttachment, XmATTACH_FORM,
			    XmNleftOffset, center_frame_left_offset,
			    XmNrightAttachment, XmATTACH_OPPOSITE_FORM,
			    XmNrightOffset, -center_frame_right_offset,
			    XmNtopAttachment, XmATTACH_FORM,
			    XmNtopOffset, title_height,
			    XmNbottomAttachment, XmATTACH_FORM,
			    XmNbottomOffset, SCAN_BOTTOM_MARGIN,
			    NULL);

  right_frame =
    XtVaCreateManagedWidget("right_frame", xmFrameWidgetClass,
			    scan_control_form,
			    XmNshadowType, XmSHADOW_IN,
			    XmNshadowThickness, SHADOW_THICKNESS,
			    XmNmarginWidth, FRAME_MARGINS,
			    XmNmarginHeight, FRAME_MARGINS,
			    XmNleftAttachment, XmATTACH_FORM,
			    XmNleftOffset, right_frame_left_offset,
			    XmNrightAttachment, XmATTACH_OPPOSITE_FORM,
			    XmNrightOffset, -right_frame_right_offset,
			    XmNtopAttachment, XmATTACH_WIDGET,
			    XmNtopWidget, left_title,
			    XmNbottomAttachment, XmATTACH_FORM,
			    XmNbottomOffset, SCAN_BOTTOM_MARGIN,
			    NULL);

  XtOverrideTranslations(left_frame, compiled_parent_translations);
  XtOverrideTranslations(center_frame, compiled_parent_translations);
  XtOverrideTranslations(right_frame, compiled_parent_translations);

  left_column =
    XtVaCreateManagedWidget("left_column", xmDrawingAreaWidgetClass,
			    left_frame,
			    XmNtranslations,
			    XtParseTranslationTable(scan_translations),
			    NULL);

  center_column =
    XtVaCreateManagedWidget("center_column", xmDrawingAreaWidgetClass,
			    center_frame,
			    XmNtranslations,
			    XtParseTranslationTable(scan_translations),
			    NULL);

  right_column =
    XtVaCreateManagedWidget("right_column", xmDrawingAreaWidgetClass,
			    right_frame,
			    XmNtranslations,
			    XtParseTranslationTable(scan_translations),
			    NULL);


  register_cb(left_column, XmNresizeCallback,
	      (CBType)&ScanControl::scan_resizeCB, NULL);

  register_cb(center_column, XmNresizeCallback,
	      (CBType)&ScanControl::scan_resizeCB, NULL);

  register_cb(right_column, XmNresizeCallback,
	      (CBType)&ScanControl::scan_resizeCB, NULL);

  // For information on position changes:
  register_eh(get_shell(center_column), StructureNotifyMask,
	      (EHType)&ScanControl::scan_configureEH, NULL);

  delete [] scan_translations;
  delete [] parent_translations;

  click_column = NO_COLUMN;
  left_ghost = False;
  right_ghost = False;

  // Register the columns as drop sites:
  static Arg args[10];
  int n = 0;

  Atom importList[1];

  importList[0] = thumbnail_atom;

  // Set the column scan_contexts to this:
  XSaveContext(dpy, XtWindow(left_column), scan_context, (XPointer)this);
  XSaveContext(dpy, XtWindow(center_column), scan_context, (XPointer)this);
  XSaveContext(dpy, XtWindow(right_column), scan_context, (XPointer)this);

  XtSetArg(args[n], XmNimportTargets, importList); n++;
  XtSetArg(args[n], XmNnumImportTargets, XtNumber(importList)); n++;
  //XtSetArg(args[n], XmNanimationStyle, XmDRAG_UNDER_SHADOW_IN); n++;
  XtSetArg(args[n], XmNdropProc, page_dropCB); n++;
  XtSetArg(args[n], XmNdropSiteOperations, XmDROP_COPY | XmDROP_MOVE); n++;

  XmDropSiteRegister(left_column, args, n);
  XmDropSiteRegister(center_column, args, n);
  XmDropSiteRegister(right_column, args, n);

  // Create a gc for the scan control:
  scan_gc = XCreateGC(dpy, RootWindowOfScreen(scrn), 0, NULL);

  XSetForeground(dpy, scan_gc, BlackPixelOfScreen(scrn));
  XSetBackground(dpy, scan_gc, WhitePixelOfScreen(scrn));
  XSetLineAttributes(dpy, scan_gc, 0, LineSolid, CapProjecting, JoinMiter);

  size_established = False;
}


void ScanControl::load_images()
{
  if (rendering_in_progress)
    {
      parent->
	write_message("DeckView: could not interrupt existing rendering process");
      return;
    }

  rendering_in_progress = True;

  // Load the bitmaps:
  bitmap_width = 0;
  bitmap_height = 0;

  int bitmap_page_width = 0, bitmap_page_height = 0;

  // Image the document:
  int stdout_desc[2], stdin_desc[2], stderr_desc[2];
  if (pipe(stdout_desc) || pipe(stdin_desc) || pipe(stderr_desc))
    perror("DeckView: could not create pipe");

  pid = fork();
  if (pid == -1)
    perror("DeckView: could not fork");
  else if (pid == 0)
    {
      // child
      close(stdout_desc[0]); // child uses descriptors[1]
      close(stdin_desc[1]);
      close(stderr_desc[0]);

      // Set stdout, stderr, stdin to the pipe
      dup2(stdin_desc[0], 0);
      dup2(stdout_desc[1], 1);
      dup2(stderr_desc[1], 2);
      close(stdin_desc[0]);
      close(stdout_desc[1]);
      close(stderr_desc[1]);

      // exec Ghostscript:
      execl(TopShell::appdata.gs_path,
	    "gs", "-sOutputFile=-", "-q", "-sDEVICE=pbmraw", "-dSAFER",
	    "-dNOPAUSE", "-dNODISPLAY",
	    NULL);

      perror("DeckView: exec failed");
      exit(1);
    }

  // parent
  close(stdin_desc[0]); // parent uses descriptors[1]
  close(stdout_desc[1]);
  close(stderr_desc[1]);

  // Use the prolog to render the pages:
  char use_stdin[] = ".stdin run\n";
  write(stdin_desc[1], use_stdin, (sizeof use_stdin) - 1);
  write(stdin_desc[1], rendering_prolog, (sizeof rendering_prolog) - 1);

  static char commands[300];
  const int multiple = TopShell::appdata.antialias ? 2 : 1;
  sprintf(commands, "%d %d setres\n(%s) ppm1run\n",
	  TopShell::appdata.thumbnail_dpi * multiple,
	  TopShell::appdata.thumbnail_dpi * multiple,
	  psfilename);

  write(stdin_desc[1], commands, strlen(commands));

  close(stdin_desc[1]);

  // Read the resulting PBM files:
  pbmfile = fdopen(stderr_desc[0], "r");
  page_num = 0;

  input_id =
    XtAppAddInput(TopShell::app, stderr_desc[0], (XtPointer)XtInputReadMask,
		  read_page, (XtPointer)this);

  for(int i = 0; i < doc->numpages; i++)
    {
      const int this_orientation =
	(orientation == OS_AUTO) ?
	  ((parent->get_page_orientation(i) == PORTRAIT) ?
	   OS_PORTRAIT : OS_LANDSCAPE) : orientation;

      Thumbnail *tn = (Thumbnail*)(thumbnails[i]);

      tn->reorient_geometry(this_orientation);

      bitmap_width = max(bitmap_width, (int)(tn->get_width()));
      bitmap_height = max(bitmap_height, (int)(tn->get_height()));
    }
}


int ScanControl::reorient(const int new_orientation)
{
  if (rendering_in_progress)
    {
      parent->write_message("Rendering in progress; could not reorient.");
      return REORIENT_ERROR;
    }

  orientation = new_orientation;

  load_images();

  // Set column geometries to reflect new thumbnail dimensions:
  const int scan_width = (3 * bitmap_width + 2 * SCAN_LEFT_RIGHT_MARGIN
			  + 2 * SCAN_GUTTER_WIDTH);

  XtVaSetValues(scan_control_frame,
		XmNrightAttachment, XmATTACH_OPPOSITE_FORM,
		XmNrightOffset,
		-(scan_width + 8 * SHADOW_THICKNESS + 6 * FRAME_MARGINS),
		NULL);

  // Positions:
  const int column_size =
    2 * SHADOW_THICKNESS + 2 * FRAME_MARGINS + bitmap_width;
  const int left_frame_left_offset = SCAN_LEFT_RIGHT_MARGIN;
  const int left_frame_right_offset = left_frame_left_offset + column_size;
  const int center_frame_left_offset =
    left_frame_right_offset + SCAN_GUTTER_WIDTH;
  const int center_frame_right_offset = center_frame_left_offset + column_size;
  const int right_frame_left_offset =
    center_frame_right_offset + SCAN_GUTTER_WIDTH;
  const int right_frame_right_offset = right_frame_left_offset + column_size;

  XtVaSetValues(left_title,
		XmNleftOffset, left_frame_left_offset,
		XmNrightOffset, -left_frame_right_offset,
		NULL);

  XtVaSetValues(page_display,
		XmNx, center_frame_left_offset,
		NULL);

  XtVaSetValues(right_title,
		XmNleftOffset, right_frame_left_offset,
		XmNrightOffset, -right_frame_right_offset,
		NULL);

  XtVaSetValues(left_frame,
		XmNshadowThickness, SHADOW_THICKNESS,
		XmNleftOffset, left_frame_left_offset,
		XmNrightOffset, -left_frame_right_offset,
		NULL);

  XtVaSetValues(center_frame,
		XmNshadowThickness, SHADOW_THICKNESS,
		XmNleftOffset, center_frame_left_offset,
		XmNrightOffset, -center_frame_right_offset,
		NULL);

  XtVaSetValues(right_frame,
		XmNshadowThickness, SHADOW_THICKNESS,
		XmNleftOffset, right_frame_left_offset,
		XmNrightOffset, -right_frame_right_offset,
		NULL);

  // Call the resize CB to set geometry member variables:
  scan_resizeCB(left_column, NULL, NULL);
  scan_resizeCB(center_column, NULL, NULL);
  scan_resizeCB(right_column, NULL, NULL);

  return REORIENT_SUCCESS;
}


void ScanControl::draw_small_border(Window dest,
				    const int x, const int y,
				    const int border_width,
				    const int border_height,
				    Boolean full_coverage)
{
  // A double line on top
  XDrawLine(dpy, dest, scan_gc, x, y, x + border_width, y);
  XDrawLine(dpy, dest, scan_gc, x + 1, y - 1, x + border_width + 1, y - 1);

  // Single line on left
  XDrawLine(dpy, dest, scan_gc, x, y, x, y + border_height);

  // Double line on right
  XDrawLine(dpy, dest, scan_gc, x + border_width, y,
	    x + border_width, y + border_height);
  XDrawLine(dpy, dest, scan_gc, x + border_width + 1, y - 1,
	    x + border_width + 1, y + border_height);

  // Draw bottom line only if necessary
  if (full_coverage)
    XDrawLine(dpy, dest, scan_gc, x, y + border_height,
	      x + border_width, y + border_height);
}




void ScanControl::repaint_column(Window dest,
				 const int start_y, const int stop_y,
				 SortedCltn &pages, Boolean clear)
{
  GC paint_gc = XCreateGC(dpy, dest, 0, NULL);

  XSetBackground(dpy, scan_gc, TopShell::appdata.dim_page_background);
  XSetForeground(dpy, paint_gc, TopShell::appdata.dim_page_background);

  // Set the clipping region:
  XRectangle rectangle;
  rectangle.x = (short) 0;
  rectangle.y = (short) start_y - 1;
  rectangle.width = (short) bitmap_width + 2;
  rectangle.height = (short) stop_y - start_y + 2;
  XSetClipRectangles(dpy, scan_gc, 0, 0, &rectangle, 1, YXBanded);
  XSetClipRectangles(dpy, paint_gc, 0, 0, &rectangle, 1, YXBanded);

  if (clear)
    XClearArea(dpy, dest, 0, start_y, bitmap_width, stop_y - start_y, False);

  // Draw the pages:
  if (pages.size() == 0) // pure-virtual-function error
    return;

  float current_y = start_y;

  int i = 0;
  while ((i < pages.size() - 1) &&
	 ((Thumbnail*)(pages[i]))->y_position + bitmap_height < start_y)
    i++;

  // i == index of first page included in region

  int j = i;
  while ((j < pages.size() - 1) &&
	 ((Thumbnail*)(pages[j + 1]))->y_position < stop_y)
    j++;

  // j == index of last page included in region

  //while (i < thumbnails.size())

  for (int count = i; count <= j; count++)
    {
      Thumbnail * const tn = (Thumbnail*)(pages[count]);
      const int this_y = tn->y_position;
      int next_y;

//       if (count == pages.size() - 1)
//  	next_y = height;
//       else
//  	next_y = ((Thumbnail*)(pages[count + 1]))->y_position;

      next_y = tn->stop_sensitive;

      int draw_height;

      /*if (&pages == &thumbnails)
	draw_height = min(next_y - this_y, tn->get_height());
      else
	draw_height = max(next_y - this_y, tn->get_height());*/
      if (&pages == &thumbnails)
	draw_height = tn->y_stop_visible - this_y;
      else if (&pages == &persistent_marks)
	draw_height = tn->left_extent - this_y;
      else if (&pages == &temporary_marks)
	draw_height = tn->right_extent - this_y;

      if (tn->selected)
	{
	  XSetBackground(dpy, scan_gc, WhitePixelOfScreen(scrn));
	  XSetForeground(dpy, paint_gc, WhitePixelOfScreen(scrn));
	}

      tn->draw_page(dest, paint_gc, scan_gc, 0, this_y, draw_height, tn->selected);

      if (tn->selected)
	{
	  XSetBackground(dpy, scan_gc, TopShell::appdata.dim_page_background);
	  XSetForeground(dpy, paint_gc, TopShell::appdata.dim_page_background);
	}

      if (draw_height >= tn->get_height() - 2)
	draw_small_border(dest, 0, this_y,
			  tn->get_width() - 2, draw_height - 1, True);
      else
	draw_small_border(dest, 0, this_y,
			  tn->get_width() - 2, draw_height - 4, False);
    }

  XSetBackground(dpy, scan_gc, WhitePixelOfScreen(scrn));

  XFreeGC(dpy, paint_gc);

  // Reset the clipping region
  rectangle.x = (short) 0;
  rectangle.y = (short) 0;
  rectangle.width = (short) width;
  rectangle.height = (short) height;
  XSetClipRectangles(dpy, scan_gc, 0, 0, &rectangle, 1, YXBanded);
}



void ScanControl::scan_exposeAP(Widget widget, XExposeEvent *,
				char *[], int)
{
  if ((!XtIsRealized(scan_control_frame))
      || (doc->numpages < 1))
    return;

  if(!size_established)
    {
      scan_resizeCB(left_column, NULL, NULL);
      scan_resizeCB(center_column, NULL, NULL);
      scan_resizeCB(right_column, NULL, NULL);
      size_established = True;
    }

  XtVaGetValues(left_column,
		XmNbackground, &left_background,
		XmNbottomShadowColor, &left_bottom_shadow,
		XmNtopShadowColor, &left_top_shadow,
		NULL);

  XtVaGetValues(right_column,
		XmNbackground, &right_background,
		XmNbottomShadowColor, &right_bottom_shadow,
		XmNtopShadowColor, &right_top_shadow,
		NULL);

  if(widget == left_column)
    repaint_column(XtWindow(left_column), 0, height,
		   persistent_marks, False);
  else if (widget == right_column)
    repaint_column(XtWindow(right_column), 0, height,
		   temporary_marks, False);
  else if (widget == center_column)
    repaint_column(XtWindow(center_column), 0, height,
		   thumbnails, False);

/*  if(widget == left_column)
    repaint_column(XtWindow(left_column), 0, height, persistent_marks, False);
  else if (widget == right_column)
    repaint_column(XtWindow(right_column), 0, height, temporary_marks, False);
  else if (widget == center_column)
    repaint_column(XtWindow(center_column), 0, height, thumbnails, False);
*/
  if ((widget == left_column) && left_ghost)
    draw_page_outline(widget, held_page);
  else if ((widget == right_column) && right_ghost)
    draw_page_outline(widget, held_page);
}


void ScanControl::recalculate_positions(const int start, const int stop)
{
  if (doc->numpages < 1)
    return;

  // Calculate page positions:
  float current_y;

  current_y = (float)start * pix_per_page;

  for(int i = start; i < min(stop, current_scan_page); i++)
    {
      Thumbnail * const mtn = (Thumbnail*)(thumbnails[i]);
      const int image_extent = min(bitmap_height, (int)pix_per_page);

      mtn->y_position = (int)current_y;
      mtn->start_sensitive = (int)current_y;
      mtn->stop_sensitive = (int)current_y + image_extent;
      mtn->y_stop_visible = (int)current_y + image_extent;
      mtn->selected = False;
      current_y += pix_per_page;
    }

  current_y = current_scan_page * pix_per_page;

  Thumbnail * const mtn = (Thumbnail*)(thumbnails[current_scan_page]);

  if ((start <= current_scan_page) && (stop >= current_scan_page))
    {
      mtn->y_position = (int)current_y;
      mtn->start_sensitive =
	(int)current_y;
      mtn->stop_sensitive =
	(int)current_y + bitmap_height/* + SCAN_EXPANDED_BOTTOM_MARGIN*/;
      mtn->y_stop_visible = (int)current_y + bitmap_height;
      mtn->selected = True;
    }

  current_y += bitmap_height + SCAN_EXPANDED_BOTTOM_MARGIN;

  for(int i = current_scan_page + 1; i <= stop; i++)
    {
      Thumbnail * const mtn = (Thumbnail*)(thumbnails[i]);
      const int image_extent = min(bitmap_height, (int)pix_per_page);

      mtn->y_position = (int)current_y;
      mtn->start_sensitive = (int)current_y;
      mtn->stop_sensitive = (int)current_y + image_extent;
      mtn->y_stop_visible = mtn->stop_sensitive;
      mtn->selected = False;
      current_y += pix_per_page;
    }

  // Now calculate the extents for the bookmarks (they may be longer)
  for (int i = 0; i < persistent_marks.size(); i++)
    {
      Thumbnail * const tn = (Thumbnail*)(persistent_marks[i]);
      const int this_y = tn->y_position;
      int next_y; // = tn->stop_sensitive;

      if (i == persistent_marks.size() - 1)
	next_y = height;
      else
	next_y = ((Thumbnail*)(persistent_marks[i + 1]))->y_position;

      tn->left_extent = min(next_y, (int)(this_y + tn->get_height()));

      if (tn->get_page_number() == current_scan_page)
	tn->left_sensitive = tn->left_extent;// + SCAN_EXPANDED_BOTTOM_MARGIN;
      else
	tn->left_sensitive = tn->left_extent;

      tn->persistent_index = i;
    }

  for (int i = 0; i < temporary_marks.size(); i++)
    {
      Thumbnail * const tn = (Thumbnail*)(temporary_marks[i]);
      const int this_y = tn->y_position;
      int next_y; // = tn->stop_sensitive;

      if (i == temporary_marks.size() - 1)
	next_y = height;
      else
	next_y = ((Thumbnail*)(temporary_marks[i + 1]))->y_position;

      tn->right_extent = min(next_y, (int)(this_y + tn->get_height()));

      if (tn->get_page_number() == current_scan_page)
	tn->right_sensitive = tn->right_extent;
      else
	tn->right_sensitive = tn->right_extent;

      tn->temporary_index = i;
    }
}


void ScanControl::scan_resizeCB(Widget widget, XtPointer,
				XmDrawingAreaCallbackStruct*)
{
  if ((doc->numpages < 1) || (!XtIsRealized(widget)))
    return;

  Dimension new_width, new_height;

  XtVaGetValues(widget,
		XmNheight, &new_height,
		XmNwidth, &new_width,
		NULL);

  // Avoid tiny windows:
  width = max(new_width, 2);
  height = max(new_height, 2);

  if (widget == left_column)
    get_root_coords(left_column, left_x, left_y);
  else if (widget == center_column)
    get_root_coords(center_column, center_x, center_y);
  else if (widget == right_column)
    get_root_coords(right_column, right_x, right_y);

  // Height available for edge-on thumbnails,
  // i.e. total - margins - height of expanded page
  int free_height =
    height - SCAN_EXPANDED_BOTTOM_MARGIN - bitmap_height;

  // Height per page for edge-on thumbnails
  // (prevent divide-by-zero errors)
  pix_per_page = (doc->numpages == 1) ?
    0 : (float)free_height / (float)(doc->numpages - 1);

  pix_per_page =
    min((float)(bitmap_height + SCAN_EXPANDED_BOTTOM_MARGIN), pix_per_page);

  // Calculate the new page positions
  recalculate_positions(0, doc->numpages - 1);

  size_established = True;

  // Repaint the display:
  if(XtIsRealized(widget))
    XClearArea(dpy, XtWindow(widget), 0, 0, 0, 0, True);
}


int ScanControl::click_to_page(const int y, SortedCltn &pages)
{
  // Perhaps cheat somewhat here and use the traditional (O(1)) method
  // when &pages == &thumbnails rather than an O(n) search.  Seek time
  // could also be improved to O(lg n) using binary search.

  if (pages.isEmpty())
    return -1;

  // Use brute force for now
  for(int i = 0; i < pages.size(); i++)
    {
      Thumbnail * const tn = (Thumbnail*)(pages[i]);

//       int thumbnail_floor;

//       if (i < pages.size() - 1)
// 	{
// 	  Thumbnail * const tn2 = (Thumbnail*)(pages[i + 1]);
// 	  thumbnail_floor = tn2->y_position;
// 	}
//       else
// 	thumbnail_floor = height;

//       const int page_height =
// 	min(thumbnail_floor - tn->y_position, tn->get_height());
      if(&pages == &thumbnails)
	{
	  if ((y >= tn->start_sensitive) && (y <= tn->stop_sensitive))
	    return tn->get_page_number();
	}
      else if (&pages == &persistent_marks)
	{
	  if ((y >= tn->start_sensitive) && (y <= tn->left_sensitive))
	    return tn->get_page_number();
	}
      else if (&pages == &temporary_marks)
	{
	  if ((y >= tn->start_sensitive) && (y <= tn->right_sensitive))
	    return tn->get_page_number();
	}

/*	{
	  int thumbnail_floor;

	  if (i < pages.size() - 1)
	    {
	      Thumbnail * const tn2 = (Thumbnail*)(pages[i + 1]);
	      thumbnail_floor = tn2->start_sensitive;
	    }
	  else
	    thumbnail_floor = height;

	  const int page_end =
	    min(thumbnail_floor, tn->start_sensitive + tn->get_height());

	  if ((y >= tn->start_sensitive) && (y <= page_end))
	    return tn->get_page_number();
	}*/
    }

  // Not found:
  return -1;
}


void ScanControl::draw_raised_outline(Window dest,
				      const int x_start, const int y_start,
				      const int x_end, const int y_end,
				      Boolean left)
{
  XSetForeground(dpy, scan_gc, left ? left_background : right_background);

  XFillRectangle(dpy, dest, scan_gc, x_start, y_start,
		 x_end - x_start, y_end - y_start);

  // Top and left sides:
  XSetForeground(dpy, scan_gc, left ? left_top_shadow :
		 right_top_shadow);

  XDrawLine(dpy, dest, scan_gc, x_start, y_start, x_start, y_end);
  XDrawLine(dpy, dest, scan_gc, x_start, y_start, x_end, y_start);

  // Bottom and right sides:
  XSetForeground(dpy, scan_gc, left ? left_bottom_shadow :
		 right_bottom_shadow);

  XDrawLine(dpy, dest, scan_gc, x_end, y_start, x_end, y_end);
  XDrawLine(dpy, dest, scan_gc, x_start, y_end, x_end, y_end);

  XSetForeground(dpy, scan_gc, BlackPixelOfScreen(scrn));
}


Boolean ScanControl::PageConvertProc(Widget dc, Atom *, Atom *target,
				     Atom *typeRtn, XtPointer *valueRtn,
				     unsigned long *lengthRtn, int *formatRtn,
				     unsigned long *, XtPointer, XtRequestId *)
{
  if (*target != thumbnail_atom)
    return(False);

  int *indices;
  XtVaGetValues(dc,
		XmNclientData, &indices,
		NULL);


  *typeRtn = thumbnail_atom;
  *valueRtn = (XtPointer)indices;
  *lengthRtn = 1;
  *formatRtn = 8 * sizeof(int);

  return(True);
}


void ScanControl::scan_inputAP(Widget widget, XEvent *event, char *args[],
			       int num_args)
{
  if (num_args != 1)
    {
      cerr << "scan_inputAP: invalid argument list" << endl;
      return;
    }

  int selected_page;

  // Well, this is no longer exactly right...

  // At this point, selected_page has the (virtual) index of the page
  // indicated by the pointer.  current_scan_page is the index of the
  // page currently displayed by the control, and hold_page is the
  // index of the selected page.  The following relationships should
  // hold after processing:
  //
  // pointer in center column:
  //   current_scan_page == hold_page == selected_page
  // pointer in left or right column:
  //   current_scan_page == hold_page
  // pointer outside of page area:
  //   current_scan_page == parent->get_page(); hold_page maintains
  //   the (invisible) selection

  if ((String("downclick") == args[0]) || (String("dragclick") == args[0]))
    {
      // Pointer click in the scan control

      if (widget == center_column)
	{
	  click_column = CENTER_COLUMN;
	  selected_page = click_to_page(event->xbutton.y, thumbnails);
	}
      else if (widget == left_column)
	{
	  click_column = LEFT_COLUMN;
	  selected_page = click_to_page(event->xbutton.y, persistent_marks);
	}
      else if (widget == right_column)
	{
	  click_column = RIGHT_COLUMN;
	  selected_page = click_to_page(event->xbutton.y, temporary_marks);
	}
      else
	{
	  click_column = NO_COLUMN;
	  return;
	}

      if (strcmp("dragclick", args[0]) == 0)
	{
	  // It's the start of a drag

	  if (selected_page == -1)
	    return;

	  Atom exportList[1];

	  exportList[0] = thumbnail_atom;

	  Widget drag_icon;

	  if (TopShell::appdata.antialias && TopShell::appdata.large_icons)
	    {
	      AntiAliasedThumbnail *ptn =
		(AntiAliasedThumbnail*)(thumbnails[held_page]);

	      if (ptn->get_active() != XmUNSPECIFIED_PIXMAP)
		drag_icon =
		  TopShell::create_drag_icon(widget, ptn->get_active(),
					     ptn->get_width(),
					     ptn->get_height());
	      else
		drag_icon =
		  TopShell::create_drag_icon(widget, thumbnail_icon_pixmap,
					     thumbnail_width,
					     thumbnail_height);
	    }
	  else
	    drag_icon =
	      TopShell::create_drag_icon(widget, thumbnail_icon_pixmap,
					 thumbnail_width, thumbnail_height);

	  static Arg args[10];
	  int n = 0, *index = new int[1];
	  index[0] = selected_page;

	  held_page = selected_page;

	  static XtCallbackRec dropFinishCB[] =
	    { 
	      {(XtCallbackProc)finish_dropCB, NULL},
	      {NULL, NULL} 
	    };

	  static XtCallbackRec enterSiteCB[] =
	    {
	      {(XtCallbackProc)enter_siteCB, NULL},
	      {NULL, NULL}
	    };

	  static XtCallbackRec leaveSiteCB[] =
	    {
	      {(XtCallbackProc)leave_siteCB, NULL},
	      {NULL, NULL}
	    };

	  static XtCallbackRec dragDropFinishCB[] =
	    {
	      {(XtCallbackProc)destroyWidgetCB, NULL},
	      {NULL, NULL}
	    };

	  static XtPointer pointers[2];

	  pointers[0] = (XtPointer)this;
	  pointers[1] = (XtPointer)widget;

	  dropFinishCB[0].closure = (XtPointer)this;
	  dragDropFinishCB[0].closure = (XtPointer)drag_icon;
	  leaveSiteCB[0].closure = (XtPointer)pointers;
	  enterSiteCB[0].closure = (XtPointer)pointers;

	  // Make the appropriate drop sites active:
	  XtSetArg(args[0], XmNdropSiteActivity, XmDROP_SITE_ACTIVE);
	  XtSetArg(args[1], XmNdropSiteOperations, XmDROP_MOVE | XmDROP_COPY);
	  XtSetArg(args[2], XmNdropSiteActivity, XmDROP_SITE_INACTIVE);
	  XtSetArg(args[3], XmNdropSiteOperations, XmDROP_NOOP);

	  XmDropSiteStartUpdate(scan_control_form);
	  switch(click_column)
	    {
	    case LEFT_COLUMN:
	      XmDropSiteUpdate(left_column, args + 2, 2);
	      XmDropSiteUpdate(center_column, args, 2);
	      XmDropSiteUpdate(right_column, args, 2);
	      break;
	    case CENTER_COLUMN:
	      XmDropSiteUpdate(left_column, args, 2);
	      XmDropSiteUpdate(center_column, args + 2, 2);
	      XmDropSiteUpdate(right_column, args, 2);
	      break;
	    case RIGHT_COLUMN:
	      XmDropSiteUpdate(left_column, args, 2);
	      XmDropSiteUpdate(center_column, args, 2);
	      XmDropSiteUpdate(right_column, args + 2, 2);
	      break;
	    default:
	      cerr << "Warning: could not find column." << endl;
	    }
	  XmDropSiteEndUpdate(scan_control_form);

	  n = 0;
	  XtSetArg(args[n], XmNexportTargets, exportList); n++;
	  XtSetArg(args[n], XmNnumExportTargets, XtNumber(exportList)); n++;
	  XtSetArg(args[n], XmNdragOperations,
		   XmDROP_COPY |
		   ((click_column == CENTER_COLUMN) ? 0 : XmDROP_MOVE)); n++;
	  XtSetArg(args[n], XmNconvertProc, PageConvertProc); n++;
	  XtSetArg(args[n], XmNclientData, (XtPointer)index); n++;
	  XtSetArg(args[n], XmNdropFinishCallback, dropFinishCB); n++;
	  XtSetArg(args[n], XmNdragDropFinishCallback, dragDropFinishCB); n++;
	  XtSetArg(args[n], XmNdropSiteEnterCallback, enterSiteCB); n++;
	  XtSetArg(args[n], XmNdropSiteLeaveCallback, leaveSiteCB); n++;
	  XtSetArg(args[n], XmNsourcePixmapIcon, drag_icon); n++;

	  Widget drag_context = XmDragStart(widget, event, args, n);

	  // Set the docwin_context of XtWindow(drag_context) to this
	  XSaveContext(dpy, XtWindow(drag_context),
		       scan_context, (XPointer)this);

	  // Prevent confusion with other events
//	  click_column = NO_COLUMN;

	  // Update the display if the page has changed:
	  if(selected_page != parent->get_current_page())
	    change_page_to(selected_page);

	  return;
	}

      // Not a drag, just a Button1 downclick:

      prev_x = event->xbutton.x_root;
      prev_y = event->xbutton.y_root;

      // Create a cursor for the grab:
      Cursor cursor = XCreateFontCursor(dpy, XC_fleur);

      XtGrabPointer(center_column, True,
		    (unsigned int)(Button1MotionMask | ButtonReleaseMask),
		    GrabModeAsync, GrabModeAsync, //None
		    XtWindow(scan_control_form), cursor, event->xbutton.time);

      XFreeCursor(dpy, cursor);

      held_page = selected_page;

      if(selected_page == -1)
	return;

      Thumbnail *this_thumbnail = (Thumbnail*)(thumbnails[selected_page]);
      const int y_offset = event->xbutton.y - this_thumbnail->start_sensitive;

      // Update the display if the page has changed:
      if(selected_page != parent->get_current_page())
	change_page_to(selected_page);

      // Warp the pointer to the corresponding position now that the
      // page may have moved:
      if (click_column != CENTER_COLUMN)
	XWarpPointer(dpy, XtWindow(widget), XtWindow(widget),
		     0, 0, 0, 0, event->xbutton.x,
		     this_thumbnail->start_sensitive + y_offset);
    }
  else if (String("upclick") == args[0])
    {
      // Pointer click in the scan control
      if (!click_column)
	return;

      left_ghost = right_ghost = False;

      int this_column;

      if (in_center_column(event->xbutton.x_root, event->xbutton.y_root))
	{
	  this_column = CENTER_COLUMN;
	  selected_page =
	    click_to_page(event->xbutton.y_root - center_y, thumbnails);
	}
      else if (in_left_column(event->xbutton.x_root, event->xbutton.y_root))
	{
	  this_column = LEFT_COLUMN;
	  selected_page =
	    click_to_page(event->xbutton.y_root - left_y, persistent_marks);
	}
      else if (in_right_column(event->xbutton.x_root, event->xbutton.y_root))
	{
	  this_column = RIGHT_COLUMN;
	  selected_page =
	    click_to_page(event->xbutton.y_root - right_y, temporary_marks);
	}
      else
	{
	  click_column = NO_COLUMN;
	  XtUngrabPointer(center_column, event->xbutton.time);
	  change_page_to(parent->get_current_page());
	  return;
	}

      const int origin_column = click_column;
      click_column = NO_COLUMN;

      // Undo the grab
      XtUngrabPointer(center_column, event->xbutton.time);

      if (held_page == -1)
	return;

      if(selected_page == -1)
	selected_page = held_page;

      // Center to center, left to left, or right to right
      if ((this_column == origin_column) &&
	  (selected_page != parent->get_current_page()))
	parent->draw_page(selected_page);

      // Center or right to left
      if ((this_column == LEFT_COLUMN) &&
	  ((origin_column == CENTER_COLUMN) ||
	   (origin_column == RIGHT_COLUMN)))
	{
	  if (held_page != parent->get_current_page())
	    parent->draw_page(held_page);
	  Thumbnail &tn = *(Thumbnail*)(thumbnails[held_page]);

	  if (persistent_marks.occurrencesOf(tn) != 0)
	    return;

	  persistent_marks.add(tn);
	  // Make this more efficient later:
	  recalculate_positions(0, thumbnails.size() - 1);

	  repaint_column(XtWindow(left_column),
			 0, height, persistent_marks, True);
	}
      // Center or left to right
      if ((this_column == RIGHT_COLUMN) &&
	  ((origin_column == CENTER_COLUMN) || (origin_column == LEFT_COLUMN)))
	{
	  if (held_page != parent->get_current_page())
	    parent->draw_page(held_page);
	  Thumbnail &tn = *(Thumbnail*)(thumbnails[held_page]);

	  if (temporary_marks.occurrencesOf(tn) != 0)
	    return;

	  temporary_marks.add(tn);
	  recalculate_positions(0, thumbnails.size() - 1);
	  repaint_column(XtWindow(right_column),
			 0, height, temporary_marks, True);
	}
      // Right to center
      if ((this_column == CENTER_COLUMN) && (origin_column == RIGHT_COLUMN))
	{
	  if (held_page != parent->get_current_page())
	    parent->draw_page(held_page);
	  Thumbnail &tn = *(Thumbnail*)(thumbnails[held_page]);

	  if (temporary_marks.occurrencesOf(tn) == 0)
	    return;

	  temporary_marks.remove(tn);
	  recalculate_positions(0, thumbnails.size() - 1);
	  repaint_column(XtWindow(right_column),
			 0, height, temporary_marks, True);
	}
      // Left to center
      if ((this_column == CENTER_COLUMN) && (origin_column == LEFT_COLUMN))
	{
	  if (held_page != parent->get_current_page())
	    parent->draw_page(held_page);
	  Thumbnail &tn = *(Thumbnail*)(thumbnails[held_page]);

	  if (persistent_marks.occurrencesOf(tn) == 0)
	    return;

	  persistent_marks.remove(tn);
	  recalculate_positions(0, thumbnails.size() - 1);
	  repaint_column(XtWindow(left_column),
			 0, height, persistent_marks, True);
	}

      click_column = NO_COLUMN;
      held_page = -1;
    }
  else if (String("motion") == args[0])
    {
      // Pointer moved with button held

      if (!click_column)
	return;

      // Prevent multiple interpretations by the different windows
      if ((event->xmotion.x_root == prev_x) &&
	  (event->xmotion.y_root == prev_y))
	return;

      int this_column;
      if (in_center_column(event->xmotion.x_root, event->xmotion.y_root))
	{
	  this_column = CENTER_COLUMN;
	  selected_page =
	    click_to_page(event->xmotion.y_root - center_y, thumbnails);
	}
      else if (in_left_column(event->xmotion.x_root, event->xmotion.y_root))
	{
	  this_column = LEFT_COLUMN;
	  selected_page =
	    click_to_page(event->xmotion.y_root - left_y, persistent_marks);
	}
      else if (in_right_column(event->xmotion.x_root, event->xmotion.y_root))
	{
	  this_column = RIGHT_COLUMN;
	  selected_page =
	    click_to_page(event->xmotion.y_root - right_y, temporary_marks);
	}
      else
	this_column = NO_COLUMN;

/*
      if ((click_column != NO_COLUMN) && (this_column != click_column))
	{
	  Atom exportList[1];

	  exportList[0] = thumbnail_atom;

	  Widget drag_icon;

	  if (TopShell::appdata.antialias && TopShell::appdata.large_icons)
	    {
	      AntiAliasedThumbnail *ptn = (AntiAliasedThumbnail*)(thumbnails[held_page]);
	      if (ptn->get_active() != XmUNSPECIFIED_PIXMAP)
		drag_icon =
		  TopShell::create_drag_icon(widget, ptn->get_active(),
					     ptn->get_width(), ptn->get_height());
	      else
		drag_icon = TopShell::create_drag_icon(widget, thumbnail_icon_pixmap,
						       thumbnail_width, thumbnail_height);
	    }
	  else
	    drag_icon = TopShell::create_drag_icon(widget, thumbnail_icon_pixmap,
						   thumbnail_width, thumbnail_height);

	  static Arg args[10];
	  int n = 0, *index = new int[1];
	  index[0] = held_page;

	  static XtCallbackRec dropFinishCB[] =
	    { 
	      {(XtCallbackProc)finish_dropCB, NULL},
	      {NULL, NULL} 
	    };

	  static XtCallbackRec enterSiteCB[] =
	    {
	      {(XtCallbackProc)enter_siteCB, NULL},
	      {NULL, NULL}
	    };

	  static XtCallbackRec leaveSiteCB[] =
	    {
	      {(XtCallbackProc)leave_siteCB, NULL},
	      {NULL, NULL}
	    };

	  static XtCallbackRec dragDropFinishCB[] =
	    {
	      {(XtCallbackProc)destroyWidgetCB, NULL},
	      {NULL, NULL}
	    };

	  static XtPointer pointers[2];

	  pointers[0] = (XtPointer)this;
	  pointers[1] = (XtPointer)widget;

	  dropFinishCB[0].closure = (XtPointer)this;
	  dragDropFinishCB[0].closure = (XtPointer)drag_icon;
	  leaveSiteCB[0].closure = (XtPointer)pointers;
	  enterSiteCB[0].closure = (XtPointer)pointers;

	  // Make the appropriate drop sites active:
	  XtSetArg(args[0], XmNdropSiteActivity, XmDROP_SITE_ACTIVE);
	  XtSetArg(args[1], XmNdropSiteOperations, XmDROP_MOVE | XmDROP_COPY);
	  XtSetArg(args[2], XmNdropSiteActivity, XmDROP_SITE_INACTIVE);
	  XtSetArg(args[3], XmNdropSiteOperations, XmDROP_NOOP);

	  XmDropSiteStartUpdate(scan_control_form);
	  switch(click_column)
	    {
	    case LEFT_COLUMN:
	      XmDropSiteUpdate(left_column, args + 2, 2);
	      XmDropSiteUpdate(center_column, args, 2);
	      XmDropSiteUpdate(right_column, args, 2);
	      break;
	    case CENTER_COLUMN:
	      XmDropSiteUpdate(left_column, args, 2);
	      XmDropSiteUpdate(center_column, args + 2, 2);
	      XmDropSiteUpdate(right_column, args, 2);
	      break;
	    case RIGHT_COLUMN:
	      XmDropSiteUpdate(left_column, args, 2);
	      XmDropSiteUpdate(center_column, args, 2);
	      XmDropSiteUpdate(right_column, args + 2, 2);
	      break;
	    default:
	      cerr << "Warning: could not find column." << endl;
	    }
	  XmDropSiteEndUpdate(scan_control_form);

	  n = 0;
	  XtSetArg(args[n], XmNexportTargets, exportList); n++;
	  XtSetArg(args[n], XmNnumExportTargets, XtNumber(exportList)); n++;
	  XtSetArg(args[n], XmNdragOperations,
		   XmDROP_COPY | ((click_column == CENTER_COLUMN) ? 0 : XmDROP_MOVE)); n++;
	  XtSetArg(args[n], XmNconvertProc, PageConvertProc); n++;
	  XtSetArg(args[n], XmNclientData, (XtPointer)index); n++;
	  XtSetArg(args[n], XmNdropFinishCallback, dropFinishCB); n++;
	  XtSetArg(args[n], XmNdragDropFinishCallback, dragDropFinishCB); n++;
	  XtSetArg(args[n], XmNdropSiteEnterCallback, enterSiteCB); n++;
	  XtSetArg(args[n], XmNdropSiteLeaveCallback, leaveSiteCB); n++;
	  XtSetArg(args[n], XmNsourcePixmapIcon, drag_icon); n++;

	  // Fake a "ButtonPress" event for the drag
	  event->type = ButtonPress;
	  event->xbutton.button = Button1;

	  Widget origin_widget;
	  switch(click_column)
	    {
	    case LEFT_COLUMN:
	      origin_widget = left_column;
	      break;
	    case CENTER_COLUMN:
	      origin_widget = center_column;
	      break;
	    case RIGHT_COLUMN:
	      origin_widget = right_column;
	      break;
	    }

	  Widget drag_context = XmDragStart(origin_widget, event, args, n);

	  // Set the docwin_context of XtWindow(drag_context) to this
	  XSaveContext(dpy, XtWindow(drag_context), scan_context, (XPointer)this);
	}
*/

      const int old_prev_x = prev_x;
      const int old_prev_y = prev_y;

      prev_x = event->xmotion.x_root;
      prev_y = event->xmotion.y_root;

      if (this_column != click_column)
	return;

      if ((selected_page == -1) && (held_page == -1))
	return;

      int new_scan_page;

      // 0 == no warp; -1 == warp up; 1 == warp down;
      // -2 == warp up w/offset 0; 2 == warp down w/offset 0

      int warp_needed = 0;

      // new_scan_page = page to display expanded
      if (!this_column)
	new_scan_page = parent->get_current_page();
      /*else if (selected_page == -1)
	new_scan_page = held_page;
      else new_scan_page = held_page = selected_page;*/
      else if (held_page == -1)
	new_scan_page = held_page = selected_page;
      else if (selected_page == held_page)
	new_scan_page = held_page;
      else if (((this_column == CENTER_COLUMN) &&
		(!in_center_column(old_prev_x, old_prev_y))) ||
	       ((this_column == LEFT_COLUMN) &&
		(!in_left_column(old_prev_x, old_prev_y))) ||
	       ((this_column == RIGHT_COLUMN) &&
		(!in_right_column(old_prev_x, old_prev_y))))
	new_scan_page = held_page = selected_page;
      else
	{
	  Thumbnail *tn = (Thumbnail*)(thumbnails[held_page]);
	  if (old_prev_y > event->xmotion.y_root)
	    {
	      // Upward motion
	      switch (click_column)
		{
		case LEFT_COLUMN:
		  new_scan_page = held_page =
		    ((Thumbnail*)
		     (persistent_marks[(tn->persistent_index == 0) ?
				       0 : tn->persistent_index - 1]))
		      ->get_page_number();
		  warp_needed = (tn->persistent_index == 0) ? -2 : -1;
		  break;
		case RIGHT_COLUMN:
		  new_scan_page = held_page =
		    ((Thumbnail*)
		     (temporary_marks[(tn->temporary_index == 0) ?
				      0 : tn->temporary_index - 1]))
		      ->get_page_number();
		  warp_needed = (tn->temporary_index == 0) ? -2 : -1;
		  break;
		case CENTER_COLUMN:
		  warp_needed = (held_page == 0) ? -2 : -1;
		  new_scan_page = held_page =
		    (held_page == 0) ? 0 : held_page - 1;
		}
	    }
	  else if (old_prev_y < event->xmotion.y_root)
	    {
	      // Downward motion
	      switch (click_column)
		{
		case LEFT_COLUMN:
		  new_scan_page = held_page =
		    ((Thumbnail*)
		     (persistent_marks[(tn->persistent_index ==
					persistent_marks.size() - 1) ?
				       tn->persistent_index :
				       tn->persistent_index + 1]))
		      ->get_page_number();
		  warp_needed =
		    (tn->persistent_index == persistent_marks.size() - 1)
		      ? 2 : 1;
		  break;
		case RIGHT_COLUMN:
		  new_scan_page = held_page =
		    ((Thumbnail*)
		     (temporary_marks[(tn->temporary_index ==
				       temporary_marks.size() - 1) ?
				      tn->temporary_index :
				      tn->temporary_index + 1]))
		      ->get_page_number();
		  warp_needed =
		    (tn->temporary_index == temporary_marks.size() -
		     1)
		      ? 2 : 1;
		  break;
		case CENTER_COLUMN:
		  warp_needed = (held_page == thumbnails.size() - 1) ? 2 : 1;
		  new_scan_page = held_page =
		    (held_page == thumbnails.size() - 1)
		      ? held_page : held_page + 1;
		}
	    }
	  else
	    new_scan_page = held_page = selected_page;
	}

      if ((new_scan_page != current_scan_page) && (new_scan_page != -1))
	change_page_to(new_scan_page);

      if (warp_needed != 0)
	{
	  Window this_win;
	  int this_x;
	  Thumbnail *tn = (Thumbnail*)(thumbnails[new_scan_page]);

	  switch (click_column)
	    {
	    case LEFT_COLUMN:
	      this_win = XtWindow(left_column);
	      this_x = left_x;
	      break;
	    case CENTER_COLUMN:
	      this_win = XtWindow(center_column);
	      this_x = center_x;
	      break;
	    case RIGHT_COLUMN:
	      this_win = XtWindow(right_column);
	      this_x = right_x;
	      break;
	    }

	  int warp_to_y;

	  if (warp_needed < 0)
	    // Warp upwards:
	    warp_to_y =
	      tn->start_sensitive +
		((warp_needed == -1) ? TopShell::appdata.jump_size : 0);
	  else
	    // Warp downwards:
	    warp_to_y =
	      tn->stop_sensitive -
		((warp_needed == 1) ? TopShell::appdata.jump_size : 0);

	  XWarpPointer(dpy, this_win, this_win, 0, 0, 0, 0,
		       event->xmotion.x_root - this_x, warp_to_y);

	  prev_x = event->xmotion.x_root - this_x;
	  prev_y = warp_to_y;
	}
    }
  else
    cerr << "scan_inputAP: invalid argument \"" << args[0] << "\"." << endl;
}



void ScanControl::change_page_to(const int new_page)
{
  const int old_scan_page = current_scan_page;
  current_scan_page = new_page;
  const int start = min(old_scan_page, current_scan_page);
  const int stop = max(old_scan_page, current_scan_page);

  int start_y, stop_y, left_stop, right_stop;

  if (start == current_scan_page)
    start_y = ((Thumbnail*)thumbnails[start])->start_sensitive;
  if (stop == current_scan_page)
    {
      stop_y = ((Thumbnail*)thumbnails[stop])->stop_sensitive;
      left_stop = ((Thumbnail*)thumbnails[stop])->left_extent;
      right_stop = ((Thumbnail*)thumbnails[stop])->right_extent;
    }

  recalculate_positions(start, stop);

  if (start == old_scan_page)
    start_y = ((Thumbnail*)thumbnails[start])->start_sensitive;
  if (stop == old_scan_page)
    {
      stop_y = ((Thumbnail*)thumbnails[stop])->stop_sensitive;
      left_stop = ((Thumbnail*)thumbnails[stop])->left_extent;
      right_stop = ((Thumbnail*)thumbnails[stop])->right_extent;
    }

  // Force the lower bound:
  start_y = max(start_y, 0);

  // Update the columns
  static char label_string[100];
  sprintf(label_string, "Page %s", doc->pages[new_page].label);
  XmString new_label = XmStringCreateSimple(label_string);

  XtVaSetValues(page_display,
		XmNlabelString, new_label,
		NULL);

  XmStringFree(new_label);

  repaint_column(XtWindow(center_column), start_y, stop_y + 2,
		 thumbnails, True);

  repaint_column(XtWindow(left_column), 0, height, persistent_marks, True);
  repaint_column(XtWindow(right_column), 0, height, temporary_marks, True);
}



void ScanControl::change_page(int page)
{
  current_scan_page = page;
  static char label_string[100];
  sprintf(label_string, "Page %s", doc->pages[page].label);
  XmString new_label = XmStringCreateSimple(label_string);

  XtVaSetValues(page_display,
		XmNlabelString, new_label,
		NULL);

  XmStringFree(new_label);

  if (size_established)
    {
      recalculate_positions(0, thumbnails.size() - 1);
      repaint_column(XtWindow(left_column), 0, height, persistent_marks, True);
      repaint_column(XtWindow(center_column), 0, height, thumbnails, True);
      repaint_column(XtWindow(right_column), 0, height, temporary_marks, True);
    }
};


void ScanControl::read_page(XtPointer client_data, int *, XtInputId *this_id)
{
  ScanControl *this_obj = (ScanControl*)client_data;
  int new_width, new_height, result;

  Thumbnail *this_thumbnail;

  if (!TopShell::appdata.antialias)
    {
      Pixmap new_bitmap;

      result = this_obj->read_pbmraw(this_obj->pbmfile, new_width, new_height,
				     new_bitmap);

      this_thumbnail = (Thumbnail*)(this_obj->thumbnails[this_obj->page_num]);

      ((BitmapThumbnail*)this_thumbnail)->set_bitmap(new_bitmap);
    }
  else
    {
      Pixmap pixmap, active;
      result = this_obj->read_pbmraw(this_obj->pbmfile, new_width, new_height,
				     pixmap, active);

      this_thumbnail = (Thumbnail*)(this_obj->thumbnails[this_obj->page_num]);

      ((AntiAliasedThumbnail*)this_thumbnail)->set_pixmaps(pixmap, active);
    }

  this_obj->refresh_thumbnail(this_thumbnail);

  this_obj->page_num++;

  if (result == PBM_ERROR)
    this_obj->parent->write_message("Error loading thumbnail images.");

  // Kill Ghostscript and clean up if this is the last page or if there
  // has been an error:
  if ((this_obj->page_num == this_obj->doc->numpages) ||
      (result == PBM_ERROR))
    {
      // kill the gs instance:
      close(fileno(this_obj->pbmfile));
      kill(this_obj->pid, SIGTERM);
      wait(0);

      this_obj->rendering_in_progress = False;

      // Unregister this input
      XtRemoveInput(*this_id);
    }
}


void ScanControl::refresh_thumbnail(Thumbnail *tn)
{
  if (XtIsRealized(center_column) && size_established)
    {
      XClearArea(dpy, XtWindow(center_column),
		 0, tn->y_position, 0, tn->y_stop_visible, True);

      if (persistent_marks.occurrencesOf(*tn) == 1)
	XClearArea(dpy, XtWindow(left_column), 0, 0, 0, 0, True);

      if (temporary_marks.occurrencesOf(*tn) == 1)
	XClearArea(dpy, XtWindow(right_column), 0, 0, 0, 0, True);
    }
}


ScanControl::~ScanControl()
{
  XtDestroyWidget(scan_control_frame);

  XFreeGC(dpy, bitmap_gc);
  XFreeGC(dpy, pixmap_gc);

  if (rendering_in_progress)
    {
      XtRemoveInput(input_id);

      // kill the gs instance:
      close(fileno(pbmfile));
      kill(pid, SIGTERM);
      wait(0);
    }
}


void ScanControl::scan_configureEH(Widget widget, XtPointer,
				   XConfigureEvent *event, Boolean*)
{
  if (event->type != ConfigureNotify)
    return;

  if ((doc->numpages < 1) || (!XtIsRealized(widget)))
    return;

  get_root_coords(left_column, left_x, left_y);
  get_root_coords(center_column, center_x, center_y);
  get_root_coords(right_column, right_x, right_y);
}


void ScanControl::finish_dropCB(Widget, ScanControl *this_obj,
				XmDropFinishCallbackStruct *cbs)
{
  if (this_obj->held_page == -1)
    return;

  // Update the display:
  if (this_obj->held_page != this_obj->parent->get_current_page())
    this_obj->parent->draw_page(this_obj->held_page);

  if ((cbs->operation == XmDROP_MOVE) &&
      (cbs->completionStatus == XmDROP_SUCCESS))
    {
      // Thumbnail was moved; remove the original copy
      Thumbnail &tn = *(Thumbnail*)(this_obj->thumbnails[this_obj->held_page]);

      if (this_obj->click_column == LEFT_COLUMN)
	{
	  if (this_obj->persistent_marks.occurrencesOf(tn) == 0)
	    return;

	  this_obj->parent->bookmarks_modified = True;

	  this_obj->persistent_marks.remove(tn);
	  this_obj->recalculate_positions(0, this_obj->thumbnails.size() - 1);
	  this_obj->repaint_column(XtWindow(this_obj->left_column),
				   0, this_obj->height,
				   this_obj->persistent_marks, True);
	}
      else if (this_obj->click_column == RIGHT_COLUMN)
	{
	  if (this_obj->temporary_marks.occurrencesOf(tn) == 0)
	    return;

	  this_obj->temporary_marks.remove(tn);
	  this_obj->recalculate_positions(0, this_obj->thumbnails.size() - 1);
	  this_obj->repaint_column(XtWindow(this_obj->right_column),
				   0, this_obj->height,
				   this_obj->temporary_marks, True);
	}
    }

  this_obj->held_page = -1;
  this_obj->click_column = NO_COLUMN;
}


void ScanControl::page_dropCB(Widget widget, XtPointer,
			      XmDropProcCallbackStruct *cbs)
{
  XmDropTransferEntryRec transferEntries[1];
  Arg args[10];
  int n = 0;

  if ((cbs->dropAction != XmDROP) || (cbs->operation == XmDROP_LINK))
    {
      XtSetArg(args[n], XmNtransferStatus, XmTRANSFER_FAILURE); n++;
    }
  else
    {
      // Retrieve the object pointer from the DragContext's userData:
      ScanControl *this_obj = NULL;

      if (cbs->dragContext &&
	  (XFindContext(dpy, XtWindow(cbs->dragContext), scan_context,
			(XPointer*)&this_obj) == 0) &&
	  this_obj)
	{
	  XtPointer *pointers = new XtPointer[2];
	  pointers[0] = (XtPointer)this_obj;
	  pointers[1] = (XtPointer)widget;
	  transferEntries[0].target = thumbnail_atom;
	  transferEntries[0].client_data = (XtPointer)pointers;
	  XtSetArg(args[n], XmNdropTransfers, transferEntries); n++;
	  XtSetArg(args[n], XmNnumDropTransfers,
		   XtNumber(transferEntries)); n++;
	  XtSetArg(args[n], XmNtransferProc, TransferProc); n++;
	}
      else
	{
	  // No context; probably a drag from somewhere else
	  XtSetArg(args[n], XmNtransferStatus, XmTRANSFER_FAILURE); n++;
	}
    }
  XmDropTransferStart(cbs->dragContext, args, n);
}


// Warning: format might in fact be int; documentation is sketchy
void ScanControl::TransferProc(Widget, XtPointer closure,
			       Atom *, Atom *type, XtPointer value,
			       unsigned long *, int*)
{
  ScanControl * const this_obj = (ScanControl*)(((XtPointer*)closure)[0]);
  Widget this_widget = (Widget)(((XtPointer*)closure)[1]);

  delete [] closure;

  if (*type == thumbnail_atom)
    {
      int transfer_page = ((int*)value)[0];

      if (this_obj->held_page != this_obj->parent->get_current_page())
	this_obj->parent->draw_page(this_obj->held_page);
      Thumbnail &tn = *(Thumbnail*)(this_obj->thumbnails[this_obj->held_page]);

      if (this_widget == this_obj->left_column)
	{
	  if (this_obj->persistent_marks.occurrencesOf(tn) != 0)
	    return;

	  this_obj->parent->bookmarks_modified = True;

	  this_obj->persistent_marks.add(tn);
	  // Make this more efficient later:
	  this_obj->recalculate_positions(0, this_obj->thumbnails.size() - 1);

	  this_obj->repaint_column(XtWindow(this_widget), 0, this_obj->height,
				   this_obj->persistent_marks, True);
	}
      else if (this_widget == this_obj->right_column)
	{
	  if (this_obj->temporary_marks.occurrencesOf(tn) != 0)
	    return;

	  this_obj->temporary_marks.add(tn);
	  // Make this more efficient later:
	  this_obj->recalculate_positions(0, this_obj->thumbnails.size() - 1);

	  this_obj->repaint_column(XtWindow(this_widget), 0, this_obj->height,
				   this_obj->temporary_marks, True);
	}
    }
}


void ScanControl::enter_siteCB(Widget, XtPointer *client_data,
			       XmDropSiteEnterCallbackStruct *cbs)
{
  ScanControl * const this_obj = (ScanControl*)(client_data[0]);
  Widget this_widget = (Widget)(client_data[1]);

  if ((cbs->operation == XmDROP_MOVE) &&
      (cbs->dropSiteStatus == XmDROP_SITE_VALID))
    {
      switch(this_obj->click_column)
	{
	case LEFT_COLUMN:
	  this_obj->left_ghost = True;
	  this_obj->draw_page_outline(this_obj->left_column,
				      this_obj->held_page);
	  break;
	case RIGHT_COLUMN:
	  this_obj->right_ghost = True;
	  this_obj->draw_page_outline(this_obj->right_column,
				      this_obj->held_page);
	  break;
	default:
	  cerr << "Warning: move attempted from center column (" <<
	    this_obj->click_column << ")." << endl;
	}
    }
}


void ScanControl::leave_siteCB(Widget, XtPointer *client_data,
			       XmDropSiteLeaveCallbackStruct *)
{
  ScanControl * const this_obj = (ScanControl*)(client_data[0]);
  Widget this_widget = (Widget)(client_data[1]);

  Thumbnail &tn = *(Thumbnail*)(this_obj->thumbnails[this_obj->held_page]);
  const int max_ghost_height =
    min((int)(tn.get_height()), (int)(this_obj->height - tn.y_position));

  if (this_obj->left_ghost)
    {
      this_obj->left_ghost = False;
      // Flip the page outline
      this_obj->repaint_column(XtWindow(this_obj->left_column),
			       tn.y_position - 1,
			       tn.y_position + max_ghost_height,
			       this_obj->persistent_marks, True);
    }
  else if (this_obj->right_ghost)
    {
      this_obj->right_ghost = False;
      // Flip the page outline
      this_obj->repaint_column(XtWindow(this_obj->right_column),
			       tn.y_position - 1,
			       tn.y_position + max_ghost_height,
			       this_obj->temporary_marks, True);
    }
}


void ScanControl::draw_page_outline(Widget widget, const int page)
{
  Thumbnail &tn = *(Thumbnail*)(thumbnails[page]);

  const int max_ghost_height =
    min((int)(tn.get_height()), (int)(height - tn.y_position));

  // Draw the page outline
  draw_raised_outline(XtWindow(widget), 0, tn.y_position, tn.get_width() - 1,
		      tn.y_position + max_ghost_height - 1, True);
}


int ScanControl::get_column_size(const int column)
{
  switch(column)
    {
    case LEFT_COLUMN:
      return persistent_marks.size();
    case CENTER_COLUMN:
      return thumbnails.size();
    case RIGHT_COLUMN:
      return temporary_marks.size();
    }
}


SortedCltn &ScanControl::get_column(int column)
{
  switch (column)
    {
    case LEFT_COLUMN:
      return persistent_marks;
    case RIGHT_COLUMN:
      return temporary_marks;
    case CENTER_COLUMN:
      return thumbnails;
    }
}
