//  This may look like C, but it is really -*- C++ -*-

#ifndef ScanControl_H
#define ScanControl_H


/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/



#include "UIObject.h"
#include "TopShell.h"
#include "Bitmap.h"
#include <nihcl/String.h>
#include <nihcl/SortedCltn.h>
#include <nihcl/OrderedCltn.h>

#include <unistd.h>

#define String X11String
#define Object X11Object

#include <Xm/Xm.h>

#undef String
#undef Object


#define SCAN_TOP_MARGIN  15
#define SCAN_BOTTOM_MARGIN  15
#define SCAN_VERT_MARGINS  (SCAN_TOP_MARGIN + SCAN_BOTTOM_MARGIN)
#define SCAN_LEFT_RIGHT_MARGIN  10
#define SCAN_EXPANDED_BOTTOM_MARGIN  5
#define SCAN_GUTTER_WIDTH  10
#define SCAN_GUTTER_MARGIN  5
#define FRAME_MARGINS  4
#define SHADOW_THICKNESS  2

#define LEFT_COLUMN  1
#define CENTER_COLUMN  2
#define RIGHT_COLUMN  3
#define NO_COLUMN  0

//  return codes for read_pbmraw():
#define PBM_SUCCESS 0
#define PBM_ERROR 1

//  return codes for reorient():
#define REORIENT_SUCCESS 0
#define REORIENT_ERROR 1


class DocWindow;
struct document;

class ScanControl: public UIObject {
  DECLARE_REDUCED_MEMBERS(ScanControl);

private:
  DocWindow *parent;
  Widget scan_control_form, scan_control_frame;
  Widget left_column, center_column, right_column;
  Widget left_frame, center_frame, right_frame;
  Widget left_title, right_title, page_display;
  struct document *doc;

  Window win;

  Pixel left_background, left_bottom_shadow, left_top_shadow;
  Pixel right_background, right_bottom_shadow, right_top_shadow;

  SortedCltn thumbnails, temporary_marks, persistent_marks;
  float pix_per_page;
  GC scan_gc, bitmap_gc, pixmap_gc;
  int current_scan_page, held_page, prev_x, prev_y;
  Boolean left_ghost, right_ghost;
  int click_column;
  int bitmap_width, bitmap_height;
  Dimension width, height;	// Should be the same for all columns
  Boolean size_established;
  int center_x, center_y, left_x, left_y, right_x, right_y;
  FILE *pbmfile;
  pid_t pid;
  int page_num;
  Boolean rendering_in_progress;
  XtInputId input_id;
  int orientation;
  char *psfilename;

  void scan_exposeAP(Widget, XExposeEvent*, char*[], int);
  void scan_resizeCB(Widget, XtPointer, XmDrawingAreaCallbackStruct*);
  void scan_inputAP(Widget, XEvent*, char*[], int);
  void scan_configureEH(Widget, XtPointer, XConfigureEvent*, Boolean*);

  static Boolean PageConvertProc(Widget, Atom*, Atom*, Atom*, XtPointer*, unsigned long*,
				 int *, unsigned long *, XtPointer, XtRequestId*);

  static void page_dropCB(Widget, XtPointer, XmDropProcCallbackStruct *);
  static void TransferProc(Widget, XtPointer, Atom *, Atom *, XtPointer,
			   unsigned long *, int*);

  static void finish_dropCB(Widget, ScanControl*, XmDropFinishCallbackStruct*);
  static void enter_siteCB(Widget, XtPointer*, XmDropSiteEnterCallbackStruct*);
  static void leave_siteCB(Widget, XtPointer*, XmDropSiteLeaveCallbackStruct*);

  inline Boolean in_column_i(const int, const int, const int);
  inline Boolean in_gutter_i(const int, const int, const int);

  Boolean in_center_or_gutters(const int x, const int y)
    { return ((y>=center_y)&&(y<=center_y+height)&&
	      (x>=left_x+width)&&(x<=right_x)); }
  Boolean in_right_gutter(const int x, const int y)
    { return in_gutter_i(x,y,1); }

  Boolean in_left_column(const int x, const int y)
    { return ((x>=left_x)&&(x<=left_x+width)&&
	      (y>=left_y)&&(y<=left_y+height)); }
  Boolean in_center_column(const int x, const int y)
    { return ((x>=center_x)&&(x<=center_x+width)&&
	      (y>=center_y)&&(y<=center_y+height)); }
  Boolean in_right_column(const int x, const int y)
    { return ((x>=right_x)&&(x<=right_x+width)&&
	      (y>=right_y)&&(y<=right_y+height)); }


  void recalculate_positions(const int, const int);
  void draw_bookmarks(Window, const int, const int, const int, const int,
		      const SortedCltn &);
  void draw_small_border(Window, const int, const int, const int,
			 const int, Boolean);
  void draw_raised_outline(Window, const int, const int, const int, const int, Boolean);
  void repaint_column(Window, const int, const int, SortedCltn &, Boolean);
  void change_page_to(const int);
  int read_pbmraw(FILE*, int&, int&, Pixmap&);
  int read_pbmraw(FILE*, int&, int&, Pixmap&, Pixmap&);

  int click_to_page(const int, SortedCltn&);
  void draw_page_outline(Widget, const int);

  static void read_page(XtPointer, int *, XtInputId*);

public:
  ScanControl(DocWindow*, Widget, document*, OrderedCltn *, const char *);
  ~ScanControl();

  void change_page(int);
  void refresh_thumbnail(Thumbnail *);

  void scan_realize()
    { win = XtWindow(scan_control_frame); }

  SortedCltn *copy_persistent_marks()
   /* { return (SortedCltn*)(persistent_marks.shallowCopy()); }*/
    { return (new SortedCltn(persistent_marks)); }

  int reorient(const int);
  void load_images();

  int get_column_size(const int);
  Widget get_top_widget() {return scan_control_frame;};

  SortedCltn &get_column(int);

public:
  bool operator==(const ScanControl& a) const
    { return (this == &a); }
  bool operator!=(const ScanControl& a) const
    { return !(*this==a); }
  virtual bool isEqual(const Object&) const;
#ifdef FULL_NIHCL
  virtual const Class* species() const
    { return &classDesc; }
#endif
};

#endif
