//  This may look like C, but it is really -*- C++ -*-

#ifndef Thumbnail_H
#define Thumbnail_H


/*
DeckView - A browser for Postscript documents.
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/



#include "Bitmap.h"
#include "ReducedObj.h"
#include "DocWindow.h"
#include <nihcl/String.h>


class Thumbnail: public ReducedObj {
  DECLARE_REDUCED_MEMBERS(Thumbnail);

protected:
  const String page_name;
  const int page_number;
  unsigned int image_width, image_height;
  int pixmaps_ready;

  static Pixmap transform_pixmap(Pixmap, int);

  struct RotatePixmapStruct {Pixmap *pm; int difference;
			     Thumbnail *this_obj; ScanControl *parent;};
  static Boolean rotate_pixmapWP(RotatePixmapStruct*);
  void do_reorient_geometry(int);

public:
  int y_position, y_stop_visible;
  int start_sensitive, stop_sensitive;
  int left_extent, right_extent;
  int left_sensitive, right_sensitive;
  int persistent_index, temporary_index;
  char selected;
  const int original_orientation;
  int orientation;

  Thumbnail(char * const, const int, const unsigned int,
	    const unsigned int, const int = OS_PORTRAIT);

  void print_page(FILE *fp)
    { fprintf(fp, "%d %s\n", page_number, (char*)(const char *)page_name); }

  int get_page_number() const { return page_number; }
  virtual void reorient(int, ScanControl*)
    { cerr << "Warning: Thumbnail::reorient called." << endl; }
  virtual void reorient_geometry(const int)
    { cerr << "Warning: Thumbnail::reorient_geometry called." << endl; }
  virtual unsigned int get_width() { return image_width; }
  virtual unsigned int get_height() { return image_height; }
  virtual void draw_page(Window, GC, GC, int, int, unsigned int, Boolean = False)
    { cerr << "Warning: Thumbnail::draw_page called." << endl; }

public:
  bool operator==(const Thumbnail& a) const
    { return (this == &a); }
  bool operator!=(const Thumbnail& a) const
    { return !(*this==a); }
  virtual int compare(const Object&) const;
  virtual bool isEqual(const Object&) const;
  //virtual void printOn(ostream& strm =cout) const;
#ifdef FULL_NIHCL
  virtual const Class* species() const
    { return &classDesc; }
#endif
};


class BitmapThumbnail: public Thumbnail {
  DECLARE_REDUCED_MEMBERS(BitmapThumbnail);

public:
  Pixmap thumbnail_bitmap;

  BitmapThumbnail(const unsigned int, const unsigned int,
		  char * const, const int, const int = OS_PORTRAIT);
  ~BitmapThumbnail() { XFreePixmap(dpy, thumbnail_bitmap); }

  void set_bitmap(Pixmap);
  virtual void reorient(int, ScanControl *);
  virtual void draw_page(Window, GC, GC, int, int, unsigned int, Boolean);
  virtual void reorient_geometry(const int);

public:
  bool operator==(const BitmapThumbnail& a) const
    { return (this == &a); }
  bool operator!=(const BitmapThumbnail& a) const
    { return !(*this==a); }
#ifdef FULL_NIHCL
  virtual const Class* species() const
    { return &classDesc; }
#endif
};


class AntiAliasedThumbnail: public Thumbnail {
  DECLARE_REDUCED_MEMBERS(AntiAliasedThumbnail);

public:
  Pixmap thumbnail_pixmap, active_pixmap;

  AntiAliasedThumbnail(const unsigned int, const unsigned int,
		       char * const, const int, const int = OS_PORTRAIT);
  ~AntiAliasedThumbnail() { XFreePixmap(dpy, thumbnail_pixmap);
			    XFreePixmap(dpy, active_pixmap); }

  void set_pixmaps(Pixmap, Pixmap);
  virtual void reorient(int, ScanControl *);
  Pixmap get_active() { return active_pixmap; }
  virtual void draw_page(Window, GC, GC, int, int, unsigned int, Boolean);
  virtual void reorient_geometry(const int);

public:
  bool operator==(const AntiAliasedThumbnail& a) const
    { return (this == &a); }
  bool operator!=(const AntiAliasedThumbnail& a) const
    { return !(*this==a); }
#ifdef FULL_NIHCL
  virtual const Class* species() const
    { return &classDesc; }
#endif
};


#endif
