#ifndef	COLLECTION_H
#define	COLLECTION_H

/*$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/Collection.h,v 3.16 92/12/19 15:38:27 sandy Exp $*/

/* Collection.h -- declarations for abstract Collection class
DeckView NIHCL subset
Version 1.0, BETA 1  (November, 1996)

This file is based on the NIH Class Library, but may incorporate some
changes from the original.  The full NIHCL is available via anonymous
FTP from alw.nih.gov, in directory /pub/NIHCL.  DeckView includes only
the small subset of the Library which is directly employed; the
distributed version is far more comprehensive.  See the README.nihcl
file for details.  The original copyright notice is retained below.

These changes are free software; you can redistribute them and/or
modify them under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

These changes are distributed in the hope that they will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Changes are:
Copyright (C) 1996  Adam Ginsburg

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu


The original file is copyrighted as follows:


	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Computer Systems Laboratory, DCRT
	National Institutes of Health
	Bethesda, MD 20892

$Log:	Collection.h,v $
 * Revision 3.16  92/12/19  15:38:27  sandy
 * Added abstract virtual reSize()
 * 
 * Revision 3.15  92/07/26  14:32:03  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.14  92/06/17  11:35:21  sandy
 * __DECCCXX compatible
 * 
 * Revision 3.13  92/05/28  15:25:30  sandy
 * correct at() overloading
 * 
 * Revision 3.12  92/04/30  15:32:20  sandy
 * added member sameContentsAs(const Collection& c)
 * 
 * Revision 3.11  91/07/16  13:23:01  kgorlen
 * Implement _reader() functions.
 * 
 * Revision 3.10  91/02/18  21:06:06  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.2  91/02/16  09:53:23  kgorlen
 * Change Collection::removeAll(const Collection&) to
 * removeCltn(const Collection&).
 * 
 * Revision 3.1  90/11/10  18:31:27  kgorlen
 * Add virtual unsigned capacity() const = 0
 * 
 * Revision 3.0  90/05/20  00:19:21  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/Object.h>

class Iterator;

class Collection: public VIRTUAL Object {	// abstract class 
	DECLARE_MEMBERS(Collection);
public:
	static const unsigned DEFAULT_CAPACITY;		// default initial collection capacity 
	static const unsigned EXPANSION_INCREMENT;	// collection (OrderedCltn) expansion increment 
	static const unsigned EXPANSION_FACTOR;		// collection (Set,Bag,Dictionary) expansion factor 
protected:
	Collection();
public:
	virtual ~Collection();		// Collection destructors are virtual
	virtual	Object* add(Object&) = 0;
	virtual const Collection& addAll(const Collection&);
	virtual Collection& addContentsTo(Collection&) const;
	virtual Object*& at(int) = 0;
#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate at() and at() const
	virtual const Object *const& at(long) const = 0;
#else
	virtual const Object *const& at(int) const = 0;
#endif
	virtual unsigned capacity() const = 0;
	virtual int compare(const Object&) const = 0;
        virtual bool sameContentsAs(const Collection& c) const;
	virtual	void doFinish(Iterator& pos) const;
	virtual	Object* doNext(Iterator&) const = 0;
	virtual	void doReset(Iterator& pos) const;
	virtual unsigned hash() const = 0;
	virtual bool includes(const Object&) const;
	virtual bool isEmpty() const;
	virtual bool isEqual(const Object&) const = 0;
	virtual unsigned occurrencesOf(const Object&) const = 0;
	virtual Object* remove(const Object&) = 0;
	virtual void removeAll() = 0;
	virtual const Collection& removeCltn(const Collection&);
        virtual void reSize(unsigned) = 0;
	virtual unsigned size() const = 0;
};

#include <nihcl/Iterator.h>

#endif
