/* Iterator.c -- Implementation of Collection Iterators
DeckView NIHCL subset
Version 1.0, BETA 1  (November, 1996)

This file is based on the NIH Class Library, but may incorporate some
changes from the original.  The full NIHCL is available via anonymous
FTP from alw.nih.gov, in directory /pub/NIHCL.  DeckView includes only
the small subset of the Library which is directly employed; the
distributed version is far more comprehensive.  See the README.nihcl
file for details.  The original copyright notice is retained below.

These changes are free software; you can redistribute them and/or
modify them under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

These changes are distributed in the hope that they will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Changes are:
Copyright (C) 1996  Adam Ginsburg

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu


The original file is copyrighted as follows:


	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	December, 1987

Function:

Instances of class Iterator are used to iterate over (i.e. sequence
through) the objects contained in a Collection.  For example:

	OrderedCltn c;
	c.add(*new Point(0,0));
	c.add(*new Point(1,1));
//	...
	Iterator i(c);
	Object* p;
	while (p = i++) cout << *p;

will print all the Point objects in the OrderedCltn c in the same
order in which they were added.
	
Iterators may be used on any derived class of Collection, and several
Iterators may be active on the same Collection at the same time
without interference.

Iterator is an NIHCL class and implements the usual Object
functionality with the following restriction: deepCopy() and
storeOn() work only for Iterators bound to classes derived from
SeqCltn.

$Log:	Iterator.c,v $
 * Revision 3.13  92/07/26  14:32:31  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.12  92/06/17  11:39:23  sandy
 * operator++() made 3.0 compatible
 * 
 * Revision 3.11  92/04/30  15:48:25  sandy
 * one initor in DEFINE_CLASS macros
 * 
 * Revision 3.10  91/02/18  21:06:50  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.0  90/05/20  00:19:55  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/Iterator.h>
#include <nihcl/Collection.h>
#include <nihcl/SeqCltn.h>

#define	THIS	Iterator
#define	BASE	Object
#define BASE_CLASSES BASE::desc()
#define MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES Object::desc()

DEFINE_CLASS(Iterator,2,"$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/Iterator.c,v 3.13 92/07/26 14:32:31 sandy Exp Locker: sandy $",NULL)

Iterator::Iterator(const Collection& c)
	: cltn(&c)
{
	index = 0;
	ptr = NULL;
	num = 0;
	state = nil;
	cltn->doReset(*this);
}

void Iterator::reset()
{
	cltn->doReset(*this);
}

Iterator::~Iterator()
{
        cltn->doFinish(*this);
}

Object* Iterator::operator++()
{
	return ptr = cltn->doNext(*this);
}

bool Iterator::operator==(const Iterator& a) const
{
	return cltn->isSame(*a.cltn)
		&& index == a.index && num == a.num
		&& state->isEqual(*a.state);
}

int Iterator::compare(const Object&) const
{
	shouldNotImplement("compare");
	return 0;
}

bool Iterator::isEqual(const Object& p) const
{
	return this==&p;
}

unsigned Iterator::hash() const
{
	//return (const unsigned)cltn ^ index ^ num ^ state->hash();
	return 0;
}
