/* Nil.c -- implementation of the nil object
DeckView NIHCL subset
Version 1.0, BETA 1  (November, 1996)

This file is based on the NIH Class Library, but may incorporate some
changes from the original.  The full NIHCL is available via anonymous
FTP from alw.nih.gov, in directory /pub/NIHCL.  DeckView includes only
the small subset of the Library which is directly employed; the
distributed version is far more comprehensive.  See the README.nihcl
file for details.  The original copyright notice is retained below.

These changes are free software; you can redistribute them and/or
modify them under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

These changes are distributed in the hope that they will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Changes are:
Copyright (C) 1996  Adam Ginsburg

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu


The original file is copyrighted as follows:


	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	September, 1985

Function:
	
Declarations and member functions for the nil object.

$Log:	Nil.c,v $
 * Revision 3.13  92/07/26  14:32:52  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.12  92/04/30  15:41:23  sandy
 * one initor in DEFINE_CLASS macros
 * remove use of NIHCL::initialize() and Class::initialize()
 * 
 * Revision 3.11  91/08/19  22:02:03  kgorlen
 * Search source file directory for errlib.h.
 * 
 * Revision 3.10  91/02/18  21:07:23  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.2  90/11/15  14:28:48  kgorlen
 * Include nihclIO.h
 * 
 * Revision 3.1  90/11/10  20:35:44  kgorlen
 * Eliminate deepCopy().
 * Move initialization of nil pointer to NIHCLInitialize class.
 * 
 * Revision 3.0  90/05/20  11:34:10  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/Object.h>

class Nil : public VIRTUAL Object {
	DECLARE_MEMBERS(Nil);
	Nil(const Nil&) {}
public:	
	Nil() {}
	virtual int compare(const Object&) const; // compare objects 
	virtual Object* copy() const;		// copy returns nil 
	virtual unsigned hash() const;		// calculate object hash 
	virtual bool isEqual(const Object&) const;	// equality test 
};

extern const int NIHCL_RDUNKCLASS;

Object*	const Object::nil = 0;	    // initialized by NIHCLInitialize::NIHCLInitialize()

#define	THIS	Nil
#define	BASE	Object
#define BASE_CLASSES BASE::desc()
#define MEMBER_CLASSES
#define VIRTUAL_BASE_CLASSES Object::desc()

_DEFINE_CLASS_ALWAYS(Nil,1,"$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/Nil.c,v 3.13 92/07/26 14:32:52 sandy Exp Locker: sandy $",NULL)
_DEFINE_CASTDOWN(Nil)

bool Nil::isEqual(const Object& ob) const { return (&ob==nil); }

unsigned Nil::hash() const { return 0; }

int Nil::compare(const Object&) const
{
	return 0;
}

Object* Nil::copy() const { return nil; }
