#ifndef	OBJECT_H
#define	OBJECT_H

/*$Header: /afs/alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/Object.h,v 3.15 92/10/14 10:03:37 sandy Exp $*/

/* Object.h -- declarations for class Object and class Class
DeckView NIHCL subset
Version 1.0, BETA 1  (November, 1996)

This file is based on the NIH Class Library, but may incorporate some
changes from the original.  The full NIHCL is available via anonymous
FTP from alw.nih.gov, in directory /pub/NIHCL.  DeckView includes only
the small subset of the Library which is directly employed; the
distributed version is far more comprehensive.  See the README.nihcl
file for details.  The original copyright notice is retained below.

These changes are free software; you can redistribute them and/or
modify them under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

These changes are distributed in the hope that they will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Changes are:
Copyright (C) 1996  Adam Ginsburg

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu


The original file is copyrighted as follows:


	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	September, 1985

Function:
	
Define class Object, the root of the class tree, and class Class, the
class that describes class objects.

$Log:	Object.h,v $
 * Revision 3.15  92/10/14  10:03:37  sandy
 * hpux compatible
 * 
 * Revision 3.14  92/09/11  14:02:32  sandy
 * g++ compatible
 * 
 * Revision 3.13  92/07/26  14:33:05  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.12  92/06/17  11:40:10  sandy
 * made __ultrix compatible
 * 
 * Revision 3.11  92/04/30  15:33:33  sandy
 * added enum error_level to class NIHCL
 * one initor argument for DEFINE_CLASS macros
 * replace Class::dictionary with Class::dictionary()
 * 
 * Revision 3.10  91/03/23  12:14:39  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.5  91/03/23  12:12:55  kgorlen
 * Add cast to Object* to readFrom() and shallowCopy() so
 * classes can be privately inherited.
 * 
 * Revision 3.4  91/02/07  11:35:31  kgorlen
 * Change argument to _deepenVBase() to const void*.
 * 
 * Revision 3.3  90/12/22  08:55:10  kgorlen
 * Change NESTED_TYPES to NO_NESTED_TYPES.
 * 
 * Revision 3.2  90/11/15  14:31:06  kgorlen
 * Remove Object I/O related members from Class;
 * inherit from ReadFromTbl and StoreOnTbl instead.
 * Modify deepenVBase() to bypass call to
 * _deepenVBase() if MI not defined.
 * Modify storeVBaseOn() to bypass call to
 * _storeVBase() if MI not defined.
 * Declare Class::_deepenVBase only if MI defined.
 * 
 * Revision 3.1  90/11/10  20:38:18  kgorlen
 * Move initialization of nil pointer from NIHCL::NIHCL()
 * to new class NIHCLInitialize so that nil pointer init
 * function is not called for every object construction.
 * Remove Class(OIOifd&) and Class(OIOin&).
 * 
 * Revision 3.0  90/05/22  10:42:35  kgorlen
 * Release for 1st edition.
 * 
*/

// c_plusplus	defined by AT&T C++ Translator R2.0 and earlier
// __cplusplus	defined by AT&T C++ Translator R2.0 and later
// __GNUG__	defined by GNU C++
// _CPLUSPLUS_	defined by Oregon C++

// define if typedefs within a class do not have class scope
//#define NO_NESTED_TYPES

#define bool int
#include <stddef.h>
#include <stdio.h>
#ifdef __GNUG__
#undef bool
#endif
#include <iostream.h>
#ifndef __GNUG__
#undef bool
#endif
//#undef bool

#ifdef MI
#define VIRTUAL virtual
#else
#define VIRTUAL
#endif

class Object;

#ifndef __GNUG__
typedef int bool;
#endif
const int YES = 1;
const int NO = 0;

// Provided versions for both signed and unsigned char in order to
// avoid compiler warnings when char defaults to unsigned char
inline signed char	ABS(signed char x)	{ return x >= 0 ? x : -x; }
inline unsigned char	ABS(unsigned char x)	{ return x; }
inline short	ABS(short x)	{ return x >= 0 ? x : -x; }
inline int	ABS(int x)	{ return x >= 0 ? x : -x; }
inline long	ABS(long x)	{ return x >= 0 ? x : -x; }
inline float	ABS(float x)	{ return x >= 0 ? x : -x; }
inline double	ABS(double x)	{ return x >= 0 ? x : -x; }

inline char	MAX(char a,char b)	{ return a >= b ? a : b; }
inline short	MAX(short a,short b)	{ return a >= b ? a : b; }
inline int	MAX(int a,int b)	{ return a >= b ? a : b; }
inline long	MAX(long a,long b)	{ return a >= b ? a : b; }
inline float	MAX(float a,float b)	{ return a >= b ? a : b; }
inline double	MAX(double a,double b)	{ return a >= b ? a : b; }
inline void*	MAX(void* a,void* b)	{ return a >= b ? a : b; }
inline unsigned char	MAX(unsigned char a, unsigned char b)	{ return a >= b ? a : b; }
inline unsigned short	MAX(unsigned short a, unsigned short b)	{ return a >= b ? a : b; }
inline unsigned int	MAX(unsigned int a, unsigned int b)	{ return a >= b ? a : b; }
inline unsigned long	MAX(unsigned long a, unsigned long b)	{ return a >= b ? a : b; }

inline char	MIN(char a,char b)	{ return a <= b ? a : b; }
inline short	MIN(short a,short b)	{ return a <= b ? a : b; }
inline int	MIN(int a,int b)	{ return a <= b ? a : b; }
inline long	MIN(long a,long b)	{ return a <= b ? a : b; }
inline float	MIN(float a,float b)	{ return a <= b ? a : b; }
inline double	MIN(double a,double b)	{ return a <= b ? a : b; }
inline void*	MIN(void* a,void* b)	{ return a <= b ? a : b; }
inline unsigned char	MIN(unsigned char a, unsigned char b)	{ return a <= b ? a : b; }
inline unsigned short	MIN(unsigned short a, unsigned short b)	{ return a <= b ? a : b; }
inline unsigned int	MIN(unsigned int a, unsigned int b)	{ return a <= b ? a : b; }
inline unsigned long	MIN(unsigned long a, unsigned long b)	{ return a <= b ? a : b; }

#define DECLARE_MEMBERS(classname) \
public: \
	DECLARE_CASTDOWN(classname) \


#define DECLARE_REDUCED_MEMBERS(classname) \
public: \
	DECLARE_CASTDOWN(classname) \


/*
The STRINGIZE preprocessor macro converts a name into a character
string.  It can use one of three possible ways to accomplish this.
ANSI C preprocessors, which define the symbol __STDC__, recognize "#".
If you define the symbol BS_NL, the name will be separated from
surrounding double quotes by "\\\n"; otherwise, the name is simply
embedded in double quotes.
*/
#if defined(mc300)
#define   BS_NL
#endif

#if defined(__STDC__) || defined(__ultrix) || defined(hpux)

#define STRINGIZE(s) #s

#else
#ifdef BS_NL

#define STRINGIZE(s) "\
s\
"

#else

#define STRINGIZE(s) "s"

#endif
#endif

#define _DEFINE_CLASS(classname) \

#define _DEFINE_ABSTRACT_CLASS(classname) \
extern const int NIHCL_RDABSTCLASS; \

#define _DEFINE_CLASS_VBASEFN(classname) \

#define _DEFINE_CLASS_ALWAYS(classname,version,identification,initor) \


#define _DEFINE_CASTDOWN(classname) \


#define DEFINE_CLASS(classname,version,identification,initor) \
_DEFINE_CLASS(classname) \
_DEFINE_CLASS_ALWAYS(classname,version,identification,initor) \
_DEFINE_CASTDOWN(classname) \

#define DEFINE_REDUCED_CLASS(classname,version,identification,initor) \
_DEFINE_CLASS(classname) \
_DEFINE_CLASS_ALWAYS(classname,version,identification,initor) \
_DEFINE_CASTDOWN(classname) \

#define DEFINE_REDUCED_ABSTRACT_CLASS(classname,version,identification,initor) \
_DEFINE_ABSTRACT_CLASS(classname) \
_DEFINE_CLASS_ALWAYS(classname,version,identification,initor) \
_DEFINE_CASTDOWN(classname) \

#define DEFINE_ABSTRACT_CLASS(classname,version,identification,initor) \
_DEFINE_ABSTRACT_CLASS(classname) \
_DEFINE_CLASS_ALWAYS(classname,version,identification,initor) \
_DEFINE_CASTDOWN(classname) \

#define DEFINE_CLASS_MI(classname,version,identification,initor) \
_DEFINE_CLASS(classname) \
_DEFINE_CLASS_ALWAYS(classname,version,identification,initor) \

#define DEFINE_ABSTRACT_CLASS_MI(classname,version,identification,initor) \
_DEFINE_ABSTRACT_CLASS(classname) \
_DEFINE_CLASS_ALWAYS(classname,version,identification,initor) \

class Object {		// abstract class
public:			// static member variables
	static Object* const nil;		// pointer to sole instance of nil object
//protected:
	Object() {}
	void derivedClassResponsibility(const char*) const;	// unimplemented virtual function 
	bool isSame(const Object& ob) const		{ return this==&ob; }
	void shouldNotImplement(const char*) const;		/* class cannot implement this function */
	virtual Object* copy() const {cerr << "Error: Object::copy called" << endl; return ((Object*)nil);}
	virtual unsigned capacity() const;			// subclass capacity 
	virtual int compare(const Object&) const = 0;		// compare objects 
	virtual bool isEqual(const Object&) const = 0;		// equality test 
	virtual unsigned size() const;				// # of objects in array/container subclass
};

#endif
