/* Stack.c -- implementation of class Stack
DeckView NIHCL subset
Version 1.0, BETA 1  (November, 1996)

This file is based on the NIH Class Library, but may incorporate some
changes from the original.  The full NIHCL is available via anonymous
FTP from alw.nih.gov, in directory /pub/NIHCL.  DeckView includes only
the small subset of the Library which is directly employed; the
distributed version is far more comprehensive.  See the README.nihcl
file for details.  The original copyright notice is retained below.

These changes are free software; you can redistribute them and/or
modify them under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

These changes are distributed in the hope that they will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Changes are:
Copyright (C) 1996  Adam Ginsburg

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu


The original file is copyrighted as follows:


	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2033
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-1111
	uucp: uunet!nih-csl!kgorlen
	Internet: kgorlen@alw.nih.gov
	October, 1985

Function:
	
Member function definitions for class Stack.

$Log:	Stack.c,v $
 * Revision 3.13  92/07/26  14:33:47  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.12  92/06/17  11:35:36  sandy
 * __DECCCXX compatible
 * 
 * Revision 3.11  92/04/30  15:49:31  sandy
 * one initor in DEFINE_CLASS macros
 * 
 * Revision 3.10  91/02/18  21:08:25  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.0  90/05/20  00:21:27  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/Stack.h>

#define	THIS	Stack
#define	BASE	SeqCltn
#define BASE_CLASSES BASE::desc()
#define MEMBER_CLASSES OrderedCltn::desc()
#define VIRTUAL_BASE_CLASSES

DEFINE_CLASS(Stack,2,"$Header: /afs/.alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/Stack.c,v 3.13 92/07/26 14:33:47 sandy Exp Locker: sandy $",NULL)

Stack::Stack(unsigned size) : contents(size) {}

Stack::Stack(const Stack& s) : contents(s.contents) {}

Object* Stack::add(Object& ob)	{ return contents.add(ob); }

Object*& Stack::at(int i)			{ return contents.at(size()-i-1); }

#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate at() and at() const
const Object *const& Stack::at(long i) const	{ return contents.at(size()-i-1); }
#else
const Object *const& Stack::at(int i) const	{ return contents.at(size()-i-1); }
#endif

unsigned Stack::capacity() const	{ return contents.capacity(); }

unsigned Stack::hash() const		{ return contents.hash(); }

bool Stack::isEmpty() const		{ return contents.size()==0; }

Object* Stack::last() const		{ return contents.last(); }

void Stack::reSize(unsigned newSize) { contents.reSize(newSize); }

void Stack::removeAll()
{
	contents.removeAll();
}

Object* Stack::removeLast()	{ return contents.removeLast(); }

unsigned Stack::size() const		{ return contents.size(); }

void Stack::atAllPut(Object&)	{ shouldNotImplement("atAllPut"); }

int Stack::indexOfSubCollection(const SeqCltn& /* cltn */, int /* start */) const
{
	shouldNotImplement("indexOfSubCollection");
	return 0;
}

Object* Stack::remove(const Object&)
{
	shouldNotImplement("remove");
	return 0;
}

void Stack::replaceFrom(int /* start */, int /* stop */, const SeqCltn&  /* replacement */, int /* startAt */)
{
	shouldNotImplement("replaceFrom");
}
