#ifndef	STACK_H
#define	STACK_H

/*$Header: /afs/alw.nih.gov/system/src/local/nihcl/311/share/lib/RCS/Stack.h,v 3.12 92/07/26 14:33:48 sandy Exp $*/

/* Stack.h -- declarations for class Stack
DeckView NIHCL subset
Version 1.0, BETA 1  (November, 1996)

This file is based on the NIH Class Library, but may incorporate some
changes from the original.  The full NIHCL is available via anonymous
FTP from alw.nih.gov, in directory /pub/NIHCL.  DeckView includes only
the small subset of the Library which is directly employed; the
distributed version is far more comprehensive.  See the README.nihcl
file for details.  The original copyright notice is retained below.

These changes are free software; you can redistribute them and/or
modify them under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

These changes are distributed in the hope that they will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Changes are:
Copyright (C) 1996  Adam Ginsburg

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu


The original file is copyrighted as follows:


	THIS SOFTWARE FITS THE DESCRIPTION IN THE U.S. COPYRIGHT ACT OF A
	"UNITED STATES GOVERNMENT WORK".  IT WAS WRITTEN AS A PART OF THE
	AUTHOR'S OFFICIAL DUTIES AS A GOVERNMENT EMPLOYEE.  THIS MEANS IT
	CANNOT BE COPYRIGHTED.  THIS SOFTWARE IS FREELY AVAILABLE TO THE
	PUBLIC FOR USE WITHOUT A COPYRIGHT NOTICE, AND THERE ARE NO
	RESTRICTIONS ON ITS USE, NOW OR SUBSEQUENTLY.

Author:
	K. E. Gorlen
	Computer Systems Laboratory, DCRT
	National Institutes of Health
	Bethesda, MD 20892

$Log:	Stack.h,v $
 * Revision 3.12  92/07/26  14:33:48  sandy
 * include directive in form '#include <nihcl/foo.h>'
 * 
 * Revision 3.11  92/06/17  11:35:37  sandy
 * __DECCCXX compatible
 * 
 * Revision 3.10  91/02/18  21:08:30  kgorlen
 * Release for 3rd printing of 1st edition.
 * 
 * Revision 3.1  90/11/10  21:48:17  kgorlen
 * Eliminate operator=().
 * 
 * Revision 3.0  90/05/20  00:21:28  kgorlen
 * Release for 1st edition.
 * 
*/

#include <nihcl/OrderedCltn.h>

class Stack: public SeqCltn {
	DECLARE_MEMBERS(Stack);
	OrderedCltn contents;
public:
	Stack(unsigned size =DEFAULT_CAPACITY);
	Stack(const Stack&);
	bool operator==(const Stack& s) const
		{ return contents == s.contents; }
	bool operator!=(const Stack& s)	const { return !(*this==s); }
	Object*& operator[](int i)	{ return contents.at(size()-i-1); }
#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate operator[]() and operator[]() const
	const Object *const& operator[](long i) const	{ return contents.at(size()-i-1); }
#else
	const Object *const& operator[](int i) const	{ return contents.at(size()-i-1); }
#endif
	void push(Object& ob)		{ contents.addLast(ob); }
	Object* pop()			{ return contents.removeLast(); }
	Object* top() const		{ return contents.last(); }
	virtual Object* add(Object& ob);
	virtual Object*& at(int i);
#ifdef __DECCXX
// Workaround __DECCXX bug failing to disambiguate at() and at() const
	virtual const Object *const& at(long i) const;
#else
	virtual const Object *const& at(int i) const;
#endif
	virtual unsigned capacity() const;
	virtual unsigned hash() const;
	virtual bool isEmpty() const;
	virtual Object* last() const;
	virtual void reSize(unsigned newSize);
	virtual void removeAll();
	virtual Object* removeLast();
	virtual unsigned size() const;
private:			    // shouldNotImplement();
	virtual void atAllPut(Object& ob);
	virtual int indexOfSubCollection(const SeqCltn& cltn, int start=0) const;
	virtual Object* remove(const Object&);
	virtual void replaceFrom(int start, int stop, const SeqCltn& replacement, int startAt =0);
};

#endif
