/*
Application resources for DeckView
Version 1.0, BETA 1

Copyright (C) 1996  Adam Ginsburg
Design by Adam Ginsburg, Joe Marks, and Stuart M. Shieber

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

Author: Adam Ginsburg
	Aiken G-20
	33 Oxford Street
	Cambridge, MA 02138 USA

Comments and bug reports to:
	deckview@eecs.harvard.edu
*/

// The application resource definitions:

static XtResource resources[] =
{
  { // The background color for non-selected thumbnail pages:
    XbNdimBackgroundColor,
    XbCDimBackgroundColor,
    XtRPixel,
    sizeof(Pixel),
    XtOffsetOf(AppData, dim_page_background),
    XtRString,
    XtDefaultBackground,
  },
  { // Lightest gray shade for anti-aliasing:
    XbNlevelOneGray,
    XbCLevelOneGray,
    XtRPixel,
    sizeof(Pixel),
    XtOffsetOf(AppData, level_one),
    XtRString,
    "gray75",
  },
  { // Middle gray shade for anti-aliasing:
    XbNlevelTwoGray,
    XbCLevelTwoGray,
    XtRPixel,
    sizeof(Pixel),
    XtOffsetOf(AppData, level_two),
    XtRString,
    "gray50",
  },
  { // Darkest gray shade for anti-aliasing:
    XbNlevelThreeGray,
    XbCLevelThreeGray,
    XtRPixel,
    sizeof(Pixel),
    XtOffsetOf(AppData, level_three),
    XtRString,
    "gray25",
  },
  { // The path to the Ghostscript interpreter:
    XbNinterpreter,
    XbCInterpreter,
    XtRString,
    sizeof(X11String),
    XtOffsetOf(AppData, gs_path),
    XtRImmediate,
    "/usr/local/bin/gs",
  },
  { // The path to Ghostview:
    XbNghostview,
    XbCGhostview,
    XtRString,
    sizeof(X11String),
    XtOffsetOf(AppData, gv_path),
    XtRImmediate,
    "/usr/local/bin/ghostview",
  },
  { // The print command:
    XbNprintCommand,
    XbCPrintCommand,
    XtRString,
    sizeof(X11String),
    XtOffsetOf(AppData, print_command),
    XtRImmediate,
#if defined(SVR4) || defined(SYSV) || defined(USG)
    "lp -dlaser",
#else
    "lpr -Plaser",
#endif
  },
  { // The file search pattern:
    XbNpattern,
    XbCPattern,
    XtRString,
    sizeof(X11String),
    XtOffsetOf(AppData, pattern),
    XtRImmediate,
    "*.ps",
  },
  { // The resolution for thumbnail page images:
    XbNthumbnailDPI,
    XbCThumbnailDPI,
    XtRInt,
    sizeof(int),
    XtOffsetOf(AppData, thumbnail_dpi),
    XtRImmediate,
    (XtPointer)10,
  },
  { // Whether to anti-alias thumbnail images:
    XbNantialias,
    XbCAntialias,
    XtRBoolean,
    sizeof(Boolean),
    XtOffsetOf(AppData, antialias),
    XtRImmediate,
    (XtPointer)True,
  },
  { // The number of pixels allotted to each page when scanning
    // through the document:
    XbNjumpSize,
    XbCJumpSize,
    XtRInt,
    sizeof(int),
    XtOffsetOf(AppData, jump_size),
    XtRImmediate,
    (XtPointer)15,
  },
  { // Whether the thumbnail image itself should be the drag icon.
    // This is possible only with anti-aliased thumbnails.
    XbNlargeDragIcons,
    XbCLargeDragIcons,
    XtRBoolean,
    sizeof(Boolean),
    XtOffsetOf(AppData, large_icons),
    XtRImmediate,
    (XtPointer)False,
  }
};
